import logging

from django.conf import settings
from django.core.mail import send_mail
from django.db.models.signals import post_save
from django.dispatch import receiver

from contact.models import Contact

logger = logging.getLogger(__name__)


@receiver(post_save, sender=Contact)
def send_email_notification(sender, instance, created, **kwargs):
    if created:
        subject = f"New Contact Form Submission from {instance.name}"
        message = (
            f"Name: {instance.name}\n"
            f"Email: {instance.email}\n"
            f"Message: {instance.message}"
        )
        from_email = settings.DEFAULT_FROM_EMAIL
        recipient_list = [admin[1] for admin in getattr(settings, "ADMINS", [])]

        if recipient_list:
            try:
                send_mail(
                    subject=subject,
                    message=message,
                    from_email=from_email,
                    recipient_list=recipient_list,
                    fail_silently=False,
                )
            except Exception as e:
                logger.error(f"Failed to send email: {e}")
        else:
            logger.warning("No admin recipient found in settings.ADMINS")



# Generated by Django 5.0.4 on 2024-04-24 12:07
import django.contrib.postgres.search
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0013_blogposttranslation_search_document_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogposttranslation",
            name="search_document",
            field=models.TextField(
                blank=True, default="", verbose_name="Search Document"
            ),
        ),
        migrations.AlterField(
            model_name="blogposttranslation",
            name="search_document_dirty",
            field=models.BooleanField(
                default=False, verbose_name="Search Document Dirty"
            ),
        ),
        migrations.AlterField(
            model_name="blogposttranslation",
            name="search_vector",
            field=django.contrib.postgres.search.SearchVectorField(
                blank=True, null=True, verbose_name="Search Vector"
            ),
        ),
        migrations.AlterField(
            model_name="blogposttranslation",
            name="search_vector_dirty",
            field=models.BooleanField(
                default=False, verbose_name="Search Vector Dirty"
            ),
        ),
    ]



# Generated by Django 5.2.3 on 2025-06-21 14:42

import core.fields.image
import django.contrib.postgres.indexes
import django.db.models.deletion
import parler.fields
import parler.models
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0023_remove_productimage_thumbnail'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='productcategory',
            name='menu_image_one',
        ),
        migrations.RemoveField(
            model_name='productcategory',
            name='menu_image_two',
        ),
        migrations.RemoveField(
            model_name='productcategory',
            name='menu_main_banner',
        ),
        migrations.AddField(
            model_name='productcategory',
            name='active',
            field=models.BooleanField(default=True, verbose_name='Active'),
        ),
        migrations.CreateModel(
            name='ProductCategoryImage',
            fields=[
                ('sort_order', models.IntegerField(editable=False, null=True, verbose_name='Sort Order')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('image', core.fields.image.ImageAndSvgField(upload_to='uploads/categories/', verbose_name='Image')),
                ('image_type', models.CharField(choices=[('MAIN', 'Main Image'), ('BANNER', 'Banner Image'), ('ICON', 'Icon Image'), ('THUMBNAIL', 'Thumbnail Image'), ('GALLERY', 'Gallery Image'), ('BACKGROUND', 'Background Image'), ('HERO', 'Hero Image'), ('FEATURE', 'Feature Image'), ('PROMOTIONAL', 'Promotional Image'), ('SEASONAL', 'Seasonal Image')], default='MAIN', max_length=20, verbose_name='Image Type')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='images', to='product.productcategory')),
            ],
            options={
                'verbose_name': 'Product Category Image',
                'verbose_name_plural': 'Product Category Images',
                'ordering': ['sort_order'],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ProductCategoryImageTranslation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Title')),
                ('alt_text', models.CharField(blank=True, max_length=255, null=True, verbose_name='Alt Text')),
                ('master', parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='product.productcategoryimage')),
            ],
            options={
                'verbose_name': 'Product Category Image Translation',
                'db_table': 'product_productcategoryimage_translation',
                'db_tablespace': '',
                'managed': True,
                'default_permissions': (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.AddIndex(
            model_name='productcategoryimage',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['created_at'], name='cat_img_created_ix'),
        ),
        migrations.AddIndex(
            model_name='productcategoryimage',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['updated_at'], name='cat_img_updated_ix'),
        ),
        migrations.AddIndex(
            model_name='productcategoryimage',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['sort_order'], name='cat_img_sort_ix'),
        ),
        migrations.AddIndex(
            model_name='productcategoryimage',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['category', 'image_type'], name='cat_img_category_type_ix'),
        ),
        migrations.AddIndex(
            model_name='productcategoryimage',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['category', 'active'], name='cat_img_category_active_ix'),
        ),
        migrations.AddIndex(
            model_name='productcategoryimage',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['image_type', 'active'], name='cat_img_type_active_ix'),
        ),
        migrations.AlterUniqueTogether(
            name='productcategoryimage',
            unique_together={('category', 'image_type')},
        ),
        migrations.AlterUniqueTogether(
            name='productcategoryimagetranslation',
            unique_together={('language_code', 'master')},
        ),
    ]



from django.urls import path
from rest_framework.urlpatterns import format_suffix_patterns

from region.views import RegionViewSet

urlpatterns = [
    path(
        "region",
        RegionViewSet.as_view({"get": "list", "post": "create"}),
        name="region-list",
    ),
    path(
        "region/<str:pk>",
        RegionViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="region-detail",
    ),
    path(
        "region/<str:pk>/get_regions_by_country_alpha_2",
        RegionViewSet.as_view({"get": "get_regions_by_country_alpha_2"}),
        name="region-get-regions-by-country-alpha-2",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



from datetime import timedelta

import pytest
from django.test import TestCase
from django.utils import timezone

from notification.enum import (
    NotificationCategoryEnum,
    NotificationKindEnum,
    NotificationPriorityEnum,
)
from notification.factories.notification import NotificationFactory
from notification.factories.user import NotificationUserFactory
from notification.managers import (
    NotificationUserQuerySet,
)
from notification.models.user import NotificationUser
from user.factories.account import UserAccountFactory


@pytest.mark.django_db
class TestNotificationUserQuerySet(TestCase):
    def setUp(self):
        self.now = timezone.now()

        self.user1 = UserAccountFactory()
        self.user2 = UserAccountFactory()

        self.order_notification = NotificationFactory(
            kind=NotificationKindEnum.INFO,
            category=NotificationCategoryEnum.ORDER,
            priority=NotificationPriorityEnum.HIGH,
            expiry_date=self.now + timedelta(days=7),
        )

        self.cart_notification = NotificationFactory(
            kind=NotificationKindEnum.SUCCESS,
            category=NotificationCategoryEnum.CART,
            priority=NotificationPriorityEnum.NORMAL,
            expiry_date=self.now + timedelta(days=7),
        )

        self.security_notification = NotificationFactory(
            kind=NotificationKindEnum.WARNING,
            category=NotificationCategoryEnum.SECURITY,
            priority=NotificationPriorityEnum.URGENT,
            expiry_date=self.now + timedelta(days=7),
        )

        self.expired_notification = NotificationFactory(
            kind=NotificationKindEnum.ERROR,
            category=NotificationCategoryEnum.SYSTEM,
            priority=NotificationPriorityEnum.CRITICAL,
            expiry_date=self.now - timedelta(days=1),
        )

        self.promo_notification = NotificationFactory(
            kind=NotificationKindEnum.INFO,
            category=NotificationCategoryEnum.PROMOTION,
            priority=NotificationPriorityEnum.LOW,
            expiry_date=self.now + timedelta(days=30),
        )

        self.seen_notification_user = NotificationUserFactory(
            user=self.user1,
            notification=self.order_notification,
            seen=True,
            seen_at=self.now - timedelta(hours=2),
        )

        self.unseen_notification_user = NotificationUserFactory(
            user=self.user1,
            notification=self.cart_notification,
            seen=False,
            seen_at=None,
        )

        self.user2_seen_notification = NotificationUserFactory(
            user=self.user2,
            notification=self.security_notification,
            seen=True,
            seen_at=self.now - timedelta(hours=1),
        )

        self.user2_unseen_notification = NotificationUserFactory(
            user=self.user2,
            notification=self.promo_notification,
            seen=False,
            seen_at=None,
        )

        self.expired_notification_user = NotificationUserFactory(
            user=self.user1,
            notification=self.expired_notification,
            seen=False,
            seen_at=None,
        )

        old_date = self.now - timedelta(days=10)
        self.old_notification_user = NotificationUserFactory(
            user=self.user1,
            notification=self.security_notification,
            seen=True,
            seen_at=old_date,
        )
        NotificationUser.objects.filter(
            id=self.old_notification_user.id
        ).update(created_at=old_date)

    def test_seen_notifications(self):
        seen_notifications = NotificationUser.objects.seen()

        self.assertIn(self.seen_notification_user, seen_notifications)
        self.assertIn(self.user2_seen_notification, seen_notifications)
        self.assertIn(self.old_notification_user, seen_notifications)
        self.assertNotIn(self.unseen_notification_user, seen_notifications)
        self.assertNotIn(self.user2_unseen_notification, seen_notifications)

    def test_unseen_notifications(self):
        unseen_notifications = NotificationUser.objects.unseen()

        self.assertIn(self.unseen_notification_user, unseen_notifications)
        self.assertIn(self.user2_unseen_notification, unseen_notifications)
        self.assertIn(self.expired_notification_user, unseen_notifications)
        self.assertNotIn(self.seen_notification_user, unseen_notifications)
        self.assertNotIn(self.user2_seen_notification, unseen_notifications)

    def test_for_user_filtering(self):
        user1_notifications = NotificationUser.objects.for_user(self.user1)
        user2_notifications = NotificationUser.objects.for_user(self.user2)

        expected_user1 = [
            self.seen_notification_user,
            self.unseen_notification_user,
            self.expired_notification_user,
            self.old_notification_user,
        ]
        for notification_user in expected_user1:
            self.assertIn(notification_user, user1_notifications)

        self.assertNotIn(self.user2_seen_notification, user1_notifications)
        self.assertNotIn(self.user2_unseen_notification, user1_notifications)

        expected_user2 = [
            self.user2_seen_notification,
            self.user2_unseen_notification,
        ]
        for notification_user in expected_user2:
            self.assertIn(notification_user, user2_notifications)

        self.assertNotIn(self.seen_notification_user, user2_notifications)

    def test_recent_notifications(self):
        recent_notifications = NotificationUser.objects.recent(days=7)

        expected_recent = [
            self.seen_notification_user,
            self.unseen_notification_user,
            self.user2_seen_notification,
            self.user2_unseen_notification,
            self.expired_notification_user,
        ]
        for notification_user in expected_recent:
            self.assertIn(notification_user, recent_notifications)

        self.assertNotIn(self.old_notification_user, recent_notifications)

    def test_by_notification_kind_filtering(self):
        info_notifications = NotificationUser.objects.by_notification_kind(
            NotificationKindEnum.INFO
        )
        warning_notifications = NotificationUser.objects.by_notification_kind(
            NotificationKindEnum.WARNING
        )

        expected_info = [
            self.seen_notification_user,
            self.user2_unseen_notification,
        ]
        for notification_user in expected_info:
            self.assertIn(notification_user, info_notifications)

        expected_warning = [
            self.user2_seen_notification,
            self.old_notification_user,
        ]
        for notification_user in expected_warning:
            self.assertIn(notification_user, warning_notifications)

    def test_by_notification_category_filtering(self):
        order_notifications = NotificationUser.objects.by_notification_category(
            NotificationCategoryEnum.ORDER
        )
        cart_notifications = NotificationUser.objects.by_notification_category(
            NotificationCategoryEnum.CART
        )
        security_notifications = (
            NotificationUser.objects.by_notification_category(
                NotificationCategoryEnum.SECURITY
            )
        )

        self.assertIn(self.seen_notification_user, order_notifications)
        self.assertIn(self.unseen_notification_user, cart_notifications)
        self.assertIn(self.user2_seen_notification, security_notifications)
        self.assertIn(self.old_notification_user, security_notifications)

    def test_by_notification_priority_filtering(self):
        high_priority = NotificationUser.objects.by_notification_priority(
            NotificationPriorityEnum.HIGH
        )
        urgent_priority = NotificationUser.objects.by_notification_priority(
            NotificationPriorityEnum.URGENT
        )
        critical_priority = NotificationUser.objects.by_notification_priority(
            NotificationPriorityEnum.CRITICAL
        )

        self.assertIn(self.seen_notification_user, high_priority)
        self.assertIn(self.user2_seen_notification, urgent_priority)
        self.assertIn(self.expired_notification_user, critical_priority)

    def test_high_priority_notifications(self):
        high_priority_notifications = NotificationUser.objects.high_priority()

        expected_high_priority = [
            self.seen_notification_user,
            self.user2_seen_notification,
            self.old_notification_user,
            self.expired_notification_user,
        ]
        for notification_user in expected_high_priority:
            self.assertIn(notification_user, high_priority_notifications)

        self.assertNotIn(
            self.unseen_notification_user, high_priority_notifications
        )
        self.assertNotIn(
            self.user2_unseen_notification, high_priority_notifications
        )

    def test_with_notification_data_prefetch(self):
        notifications_with_data = (
            NotificationUser.objects.with_notification_data()
        )

        self.assertGreater(len(notifications_with_data), 0)
        for notification_user in notifications_with_data:
            self.assertIsNotNone(notification_user.notification)
            self.assertIsNotNone(notification_user.notification.kind)

    def test_with_user_data_prefetch(self):
        notifications_with_user_data = NotificationUser.objects.with_user_data()

        self.assertGreater(len(notifications_with_user_data), 0)
        for notification_user in notifications_with_user_data:
            self.assertIsNotNone(notification_user.user)
            self.assertIsNotNone(notification_user.user.email)

    def test_optimized_for_list(self):
        optimized_notifications = NotificationUser.objects.optimized_for_list()

        self.assertGreater(len(optimized_notifications), 0)
        for notification_user in optimized_notifications:
            self.assertIsNotNone(notification_user.user)
            self.assertIsNotNone(notification_user.notification)

    def test_active_notifications(self):
        active_notifications = NotificationUser.objects.active_notifications()

        expected_active = [
            self.seen_notification_user,
            self.unseen_notification_user,
            self.user2_seen_notification,
            self.user2_unseen_notification,
            self.old_notification_user,
        ]
        for notification_user in expected_active:
            self.assertIn(notification_user, active_notifications)

        self.assertNotIn(self.expired_notification_user, active_notifications)

    def test_expired_notifications(self):
        expired_notifications = NotificationUser.objects.expired_notifications()

        self.assertIn(self.expired_notification_user, expired_notifications)

        active_notification_users = [
            self.seen_notification_user,
            self.unseen_notification_user,
            self.user2_seen_notification,
            self.user2_unseen_notification,
        ]
        for notification_user in active_notification_users:
            self.assertNotIn(notification_user, expired_notifications)

    def test_seen_today(self):
        current_time = timezone.now()

        notification1 = NotificationFactory(
            kind=NotificationKindEnum.INFO,
            category=NotificationCategoryEnum.ORDER,
            priority=NotificationPriorityEnum.HIGH,
            expiry_date=current_time + timedelta(days=7),
        )

        notification2 = NotificationFactory(
            kind=NotificationKindEnum.SUCCESS,
            category=NotificationCategoryEnum.CART,
            priority=NotificationPriorityEnum.NORMAL,
            expiry_date=current_time + timedelta(days=7),
        )

        new_seen_user = NotificationUserFactory(
            user=self.user1,
            notification=notification1,
            seen=True,
            seen_at=current_time,
        )

        new_seen_user2 = NotificationUserFactory(
            user=self.user2,
            notification=notification2,
            seen=True,
            seen_at=current_time,
        )

        today_seen = NotificationUser.objects.seen_today()

        self.assertNotIn(self.old_notification_user, today_seen)
        self.assertNotIn(self.unseen_notification_user, today_seen)

        self.assertIn(new_seen_user, today_seen)
        self.assertIn(new_seen_user2, today_seen)

        self.assertGreaterEqual(today_seen.count(), 2)

    def test_by_date_range_filtering(self):
        start_date = (self.now - timedelta(days=1)).date()
        end_date = (self.now + timedelta(days=1)).date()

        date_range_notifications = NotificationUser.objects.by_date_range(
            start_date, end_date
        )

        expected_in_range = [
            self.seen_notification_user,
            self.unseen_notification_user,
            self.user2_seen_notification,
            self.user2_unseen_notification,
            self.expired_notification_user,
        ]
        for notification_user in expected_in_range:
            self.assertIn(notification_user, date_range_notifications)

        self.assertNotIn(self.old_notification_user, date_range_notifications)

    def test_category_specific_filters(self):
        order_notifications = NotificationUser.objects.order_related()
        cart_notifications = NotificationUser.objects.cart_related()
        security_notifications = NotificationUser.objects.security_related()
        promotional_notifications = NotificationUser.objects.promotional()

        self.assertIn(self.seen_notification_user, order_notifications)
        self.assertIn(self.unseen_notification_user, cart_notifications)
        self.assertIn(self.user2_seen_notification, security_notifications)
        self.assertIn(self.user2_unseen_notification, promotional_notifications)

    def test_chained_filtering(self):
        user1_unseen = NotificationUser.objects.for_user(self.user1).unseen()

        expected_user1_unseen = [
            self.unseen_notification_user,
            self.expired_notification_user,
        ]
        for notification_user in expected_user1_unseen:
            self.assertIn(notification_user, user1_unseen)

        self.assertNotIn(self.seen_notification_user, user1_unseen)
        self.assertNotIn(self.old_notification_user, user1_unseen)

    def test_complex_filtering_combinations(self):
        user1_active_unseen_high_priority = (
            NotificationUser.objects.for_user(self.user1)
            .unseen()
            .active_notifications()
            .high_priority()
        )

        self.assertEqual(len(user1_active_unseen_high_priority), 0)


@pytest.mark.django_db
class TestNotificationUserManager(TestCase):
    def setUp(self):
        self.now = timezone.now()

        self.user1 = UserAccountFactory()
        self.user2 = UserAccountFactory()

        self.notification1 = NotificationFactory(
            kind=NotificationKindEnum.INFO,
            priority=NotificationPriorityEnum.HIGH,
        )

        self.notification2 = NotificationFactory(
            kind=NotificationKindEnum.SUCCESS,
            priority=NotificationPriorityEnum.NORMAL,
        )

        self.notification3 = NotificationFactory(
            kind=NotificationKindEnum.WARNING,
            priority=NotificationPriorityEnum.URGENT,
        )

        self.seen_notification_user = NotificationUserFactory(
            user=self.user1,
            notification=self.notification1,
            seen=True,
            seen_at=self.now - timedelta(hours=2),
        )

        self.unseen_notification_user1 = NotificationUserFactory(
            user=self.user1,
            notification=self.notification2,
            seen=False,
            seen_at=None,
        )

        self.unseen_notification_user2 = NotificationUserFactory(
            user=self.user1,
            notification=self.notification3,
            seen=False,
            seen_at=None,
        )

        self.user2_notification = NotificationUserFactory(
            user=self.user2,
            notification=self.notification1,
            seen=False,
            seen_at=None,
        )

    def test_manager_delegation_methods(self):
        seen_notifications = NotificationUser.objects.seen()
        unseen_notifications = NotificationUser.objects.unseen()
        user1_notifications = NotificationUser.objects.for_user(self.user1)

        self.assertIn(self.seen_notification_user, seen_notifications)
        self.assertIn(self.unseen_notification_user1, unseen_notifications)
        self.assertIn(self.seen_notification_user, user1_notifications)

    def test_bulk_mark_seen_for_user(self):
        self.assertFalse(self.unseen_notification_user1.seen)
        self.assertFalse(self.unseen_notification_user2.seen)
        self.assertFalse(self.user2_notification.seen)

        updated_count = NotificationUser.objects.bulk_mark_seen(self.user1)

        self.assertEqual(updated_count, 2)

        self.unseen_notification_user1.refresh_from_db()
        self.unseen_notification_user2.refresh_from_db()
        self.user2_notification.refresh_from_db()

        self.assertTrue(self.unseen_notification_user1.seen)
        self.assertTrue(self.unseen_notification_user2.seen)
        self.assertIsNotNone(self.unseen_notification_user1.seen_at)
        self.assertIsNotNone(self.unseen_notification_user2.seen_at)

        self.assertFalse(self.user2_notification.seen)
        self.assertIsNone(self.user2_notification.seen_at)

        self.seen_notification_user.refresh_from_db()
        self.assertTrue(self.seen_notification_user.seen)

    def test_bulk_mark_seen_specific_notifications(self):
        notification_ids = [self.notification2.id]
        updated_count = NotificationUser.objects.bulk_mark_seen(
            self.user1, notification_ids=notification_ids
        )

        self.assertEqual(updated_count, 1)

        self.unseen_notification_user1.refresh_from_db()
        self.unseen_notification_user2.refresh_from_db()

        self.assertTrue(self.unseen_notification_user1.seen)
        self.assertFalse(self.unseen_notification_user2.seen)

    def test_bulk_mark_seen_no_unseen_notifications(self):
        user3 = UserAccountFactory()
        NotificationUserFactory(
            user=user3,
            notification=self.notification1,
            seen=True,
            seen_at=self.now,
        )

        updated_count = NotificationUser.objects.bulk_mark_seen(user3)

        self.assertEqual(updated_count, 0)

    def test_bulk_mark_seen_nonexistent_notification_ids(self):
        fake_notification_ids = [99999, 99998]
        updated_count = NotificationUser.objects.bulk_mark_seen(
            self.user1, notification_ids=fake_notification_ids
        )

        self.assertEqual(updated_count, 0)

        self.unseen_notification_user1.refresh_from_db()
        self.unseen_notification_user2.refresh_from_db()
        self.assertFalse(self.unseen_notification_user1.seen)
        self.assertFalse(self.unseen_notification_user2.seen)

    def test_manager_queryset_type(self):
        queryset = NotificationUser.objects.get_queryset()
        self.assertIsInstance(queryset, NotificationUserQuerySet)

    def test_unseen_manager(self):
        unseen_notifications = NotificationUser.unseen_objects.all()

        expected_unseen = [
            self.unseen_notification_user1,
            self.unseen_notification_user2,
            self.user2_notification,
        ]
        for notification_user in expected_unseen:
            self.assertIn(notification_user, unseen_notifications)

        self.assertNotIn(self.seen_notification_user, unseen_notifications)

    def test_complex_manager_operations(self):
        unseen_high_priority = (
            NotificationUser.objects.for_user(self.user1)
            .unseen()
            .high_priority()
        )

        self.assertIn(self.unseen_notification_user2, unseen_high_priority)
        self.assertNotIn(self.unseen_notification_user1, unseen_high_priority)
        self.assertNotIn(self.seen_notification_user, unseen_high_priority)



from django.db import models
from django.utils.translation import gettext_lazy as _


class OrderStatus(models.TextChoices):
    PENDING = "PENDING", _("Pending")
    PROCESSING = "PROCESSING", _("Processing")
    SHIPPED = "SHIPPED", _("Shipped")
    DELIVERED = "DELIVERED", _("Delivered")
    COMPLETED = "COMPLETED", _("Completed")
    CANCELED = "CANCELED", _("Canceled")
    RETURNED = "RETURNED", _("Returned")
    REFUNDED = "REFUNDED", _("Refunded")

    @classmethod
    def get_active_statuses(cls) -> list["OrderStatus"]:
        return [cls.PENDING, cls.PROCESSING, cls.SHIPPED, cls.DELIVERED]

    @classmethod
    def get_final_statuses(cls) -> list["OrderStatus"]:
        return [cls.COMPLETED, cls.CANCELED, cls.REFUNDED]

    @classmethod
    def get_status_groups(cls) -> dict[str, list["OrderStatus"]]:
        return {
            "active": cls.get_active_statuses(),
            "final": cls.get_final_statuses(),
            "in_transit": [cls.SHIPPED],
            "needs_processing": [cls.PENDING, cls.PROCESSING],
        }


class PaymentStatus(models.TextChoices):
    PENDING = "PENDING", _("Pending")
    PROCESSING = "PROCESSING", _("Processing")
    COMPLETED = "COMPLETED", _("Completed")
    FAILED = "FAILED", _("Failed")
    REFUNDED = "REFUNDED", _("Refunded")
    PARTIALLY_REFUNDED = "PARTIALLY_REFUNDED", _("Partially Refunded")
    CANCELED = "CANCELED", _("Canceled")



from django.db import models
from django.utils.translation import gettext_lazy as _


class PayWayEnum(models.TextChoices):
    CREDIT_CARD = "CREDIT_CARD", _("Credit Card")
    PAY_ON_DELIVERY = "PAY_ON_DELIVERY", _("Pay On Delivery")
    PAY_ON_STORE = "PAY_ON_STORE", _("Pay On Store")
    PAY_PAL = "PAY_PAL", _("PayPal")
    STRIPE = "STRIPE", _("Stripe")
    BANK_TRANSFER = "BANK_TRANSFER", _("Bank Transfer")
    APPLE_PAY = "APPLE_PAY", _("Apple Pay")
    GOOGLE_PAY = "GOOGLE_PAY", _("Google Pay")

    @classmethod
    def get_online_payments(cls) -> list[str]:
        return [
            cls.CREDIT_CARD,
            cls.PAY_PAL,
            cls.STRIPE,
            cls.APPLE_PAY,
            cls.GOOGLE_PAY,
        ]

    @classmethod
    def get_offline_payments(cls) -> list[str]:
        return [
            cls.PAY_ON_DELIVERY,
            cls.PAY_ON_STORE,
            cls.BANK_TRANSFER,
        ]

    @classmethod
    def get_digital_wallet_payments(cls) -> list[str]:
        return [
            cls.APPLE_PAY,
            cls.GOOGLE_PAY,
            cls.PAY_PAL,
        ]



import importlib

from django.apps import apps
from django.core.exceptions import ImproperlyConfigured

_factory_cache = {}


def get_or_create_instance(
    app_label, model_name, factory_module_path, factory_class_name
):
    try:
        Model = apps.get_model(app_label, model_name)
    except LookupError as err:
        raise ImproperlyConfigured(
            f"Model '{model_name}' not found in app '{app_label}'."
        ) from err

    if Model.objects.exists():
        return Model.objects.order_by("?").first()
    else:
        cache_key = f"{factory_module_path}.{factory_class_name}"
        if cache_key in _factory_cache:
            factory_class = _factory_cache[cache_key]
        else:
            try:
                factory_module = importlib.import_module(factory_module_path)
                factory_class = getattr(factory_module, factory_class_name)
                _factory_cache[cache_key] = factory_class
            except ModuleNotFoundError as err:
                raise ImproperlyConfigured(
                    f"Factory module '{factory_module_path}' not found."
                ) from err
            except AttributeError as err:
                raise ImproperlyConfigured(
                    f"Factory class '{factory_class_name}' not found in '{factory_module_path}'."
                ) from err

        return factory_class.create()



from django.conf import settings
from django.contrib.auth import get_user_model
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase

from product.factories.category import ProductCategoryFactory
from product.models.category import ProductCategory
from product.serializers.category import (
    ProductCategoryDetailSerializer,
)

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE
User = get_user_model()


class ProductCategoryViewSetTestCase(APITestCase):
    def setUp(self):
        self.category = ProductCategoryFactory()
        self.sub_category = ProductCategoryFactory(parent=self.category)
        self.user = User.objects.create_user(
            email="test@example.com",
            username="testuser",
            password="testpass123",
        )

    def get_product_category_detail_url(self, pk):
        return reverse("product-category-detail", args=[pk])

    def get_product_category_list_url(self):
        return reverse("product-category-list")

    def test_list_uses_correct_serializer(self):
        url = self.get_product_category_list_url()
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("results", response.data)

        if response.data["results"]:
            category_data = response.data["results"][0]
            expected_fields = {
                "id",
                "slug",
                "translations",
                "parent",
                "active",
                "level",
                "tree_id",
                "created_at",
                "updated_at",
                "uuid",
                "recursive_product_count",
            }
            self.assertTrue(expected_fields.issubset(set(category_data.keys())))

    def test_retrieve_uses_correct_serializer(self):
        url = self.get_product_category_detail_url(self.category.id)
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_fields = {
            "id",
            "slug",
            "translations",
            "parent",
            "active",
            "level",
            "tree_id",
            "created_at",
            "updated_at",
            "uuid",
            "recursive_product_count",
            "children",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

    def test_create_request_response_serializers(self):
        payload = {
            "slug": "new-category-enhanced",
            "parent": self.category.id,
            "translations": {
                default_language: {
                    "name": "New Enhanced Category",
                    "description": "New Enhanced Category Description",
                },
            },
        }

        url = self.get_product_category_list_url()
        response = self.client.post(url, data=payload, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        expected_fields = {
            "id",
            "slug",
            "translations",
            "parent",
            "active",
            "level",
            "tree_id",
            "created_at",
            "updated_at",
            "uuid",
            "recursive_product_count",
            "children",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

        category = ProductCategory.objects.get(id=response.data["id"])
        self.assertEqual(category.slug, "new-category-enhanced")
        self.assertEqual(category.parent.id, self.category.id)

    def test_update_request_response_serializers(self):
        payload = {
            "slug": "updated-category-enhanced",
            "translations": {
                default_language: {
                    "name": "Updated Enhanced Category",
                    "description": "Updated Enhanced Category Description",
                },
            },
        }

        url = self.get_product_category_detail_url(self.category.id)
        response = self.client.put(url, data=payload, format="json")

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_fields = {
            "id",
            "slug",
            "translations",
            "parent",
            "active",
            "level",
            "tree_id",
            "created_at",
            "updated_at",
            "uuid",
            "recursive_product_count",
            "children",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

        category = ProductCategory.objects.get(id=response.data["id"])
        self.assertEqual(category.slug, "updated-category-enhanced")

    def test_partial_update_request_response_serializers(self):
        payload = {
            "slug": "partial-updated-category-enhanced",
        }

        url = self.get_product_category_detail_url(self.category.id)
        response = self.client.patch(url, data=payload, format="json")

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_fields = {
            "id",
            "slug",
            "translations",
            "parent",
            "active",
            "level",
            "tree_id",
            "created_at",
            "updated_at",
            "uuid",
            "recursive_product_count",
            "children",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

        category = ProductCategory.objects.get(id=response.data["id"])
        self.assertEqual(category.slug, "partial-updated-category-enhanced")

    def test_filtering_functionality(self):
        parent_category = ProductCategoryFactory()
        child_category = ProductCategoryFactory(parent=parent_category)

        url = self.get_product_category_list_url()
        response = self.client.get(url, {"parent": parent_category.id})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["id"], child_category.id)

        response = self.client.get(url, {"slug": self.category.slug})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["id"], self.category.id)

    def test_search_functionality(self):
        url = self.get_product_category_list_url()
        search_value = (
            self.category.safe_translation_getter("name", any_language=True)
            or ""
        )
        response = self.client.get(
            url,
            {"search": search_value},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        category_ids = [cat["id"] for cat in response.data["results"]]
        self.assertIn(self.category.id, category_ids)

    def test_ordering_functionality(self):
        url = self.get_product_category_list_url()

        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.client.get(url, {"ordering": "id"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        if len(response.data["results"]) > 1:
            ids = [cat["id"] for cat in response.data["results"]]
            self.assertEqual(ids, sorted(ids))

    def test_validation_errors_consistent(self):
        payload = {
            "slug": "invalid-parent-category",
            "parent": 99999,
            "translations": {
                default_language: {
                    "name": "Invalid Parent Category",
                    "description": "Category with invalid parent",
                },
            },
        }

        url = self.get_product_category_list_url()
        response = self.client.post(url, data=payload, format="json")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("parent", response.data)

    def test_consistency_with_manual_serializer_instantiation(self):
        url = self.get_product_category_detail_url(self.category.id)
        viewset_response = self.client.get(url)

        manual_serializer = ProductCategoryDetailSerializer(self.category)

        self.assertEqual(viewset_response.status_code, status.HTTP_200_OK)

        key_fields = ["id", "slug", "parent"]
        for field in key_fields:
            self.assertEqual(
                viewset_response.data[field], manual_serializer.data[field]
            )

    def test_create_with_complex_payload(self):
        payload = {
            "slug": "complex-category",
            "parent": self.category.id,
            "translations": {
                "en": {
                    "name": "Complex Category EN",
                    "description": "Complex Category Description EN",
                },
                "de": {
                    "name": "Complex Category DE",
                    "description": "Complex Category Description DE",
                },
            },
        }

        url = self.get_product_category_list_url()
        response = self.client.post(url, data=payload, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        category = ProductCategory.objects.get(id=response.data["id"])

        category.set_current_language("en")
        self.assertEqual(category.name, "Complex Category EN")

        category.set_current_language("de")
        self.assertEqual(category.name, "Complex Category DE")

    def test_hierarchy_data_in_responses(self):
        url = self.get_product_category_detail_url(self.category.id)
        response = self.client.get(url)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("children", response.data)
        self.assertIn("recursive_product_count", response.data)

        if response.data["children"]:
            child_data = response.data["children"][0]
            self.assertIn("id", child_data)
            self.assertIn("slug", child_data)
            self.assertIn("translations", child_data)



# Generated by Django 5.0.6 on 2024-07-07 14:38
import uuid

import django.contrib.postgres.indexes
import django.db.models.deletion
import django.db.models.manager
import parler.fields
import parler.models
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "sort_order",
                    models.IntegerField(
                        db_index=True,
                        editable=False,
                        null=True,
                        verbose_name="Sort Order",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("active", models.BooleanField(default=True, verbose_name="Active")),
            ],
            options={
                "verbose_name": "Blog Tag",
                "verbose_name_plural": "Blog Tags",
                "ordering": ["sort_order"],
                "indexes": [
                    django.contrib.postgres.indexes.BTreeIndex(
                        fields=["created_at"], name="tag_created_at_idx"
                    ),
                    django.contrib.postgres.indexes.BTreeIndex(
                        fields=["updated_at"], name="tag_updated_at_idx"
                    ),
                    django.contrib.postgres.indexes.BTreeIndex(
                        fields=["sort_order"], name="tag_sort_order_idx"
                    ),
                    django.contrib.postgres.indexes.BTreeIndex(
                        fields=["active"], name="tag_tag_active_5d272a_btree"
                    ),
                ],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
            managers=[
                ("active_tags", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="TaggedItem",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("object_id", models.PositiveIntegerField()),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "tag",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="tag.tag"
                    ),
                ),
            ],
            options={
                "verbose_name": "Tagged Item",
                "verbose_name_plural": "Tagged Items",
                "indexes": [
                    models.Index(
                        fields=["content_type", "object_id"],
                        name="tag_taggedi_content_cd6f88_idx",
                    )
                ],
            },
        ),
        migrations.CreateModel(
            name="TagTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="Label"
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="tag.tag",
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog Tag Translation",
                "db_table": "tag_tag_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
    ]



from datetime import timedelta
from unittest import TestCase
from unittest.mock import MagicMock, Mock, PropertyMock, patch

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from djmoney.money import Money

from order.enum.document_type import OrderDocumentTypeEnum
from order.enum.status import OrderStatus, PaymentStatus
from order.managers.order import OrderManager, OrderQuerySet
from order.models.order import Order


class OrderModelTestCase(TestCase):
    def setUp(self):
        self.order = Mock(spec=Order)
        self.order.id = 1
        self.order.uuid = "test-uuid-1234"
        self.order.email = "customer@example.com"
        self.order.first_name = "John"
        self.order.last_name = "Doe"
        self.order.status = OrderStatus.PENDING
        self.order.created_at = timezone.now()
        self.order.document_type = OrderDocumentTypeEnum.RECEIPT
        self.order.shipping_price = Money("10.00", settings.DEFAULT_CURRENCY)
        self.order.paid_amount = Money("0.00", settings.DEFAULT_CURRENCY)
        self.order._original_status = OrderStatus.PENDING

        self.order.street = "Test Street"
        self.order.street_number = "123"
        self.order.city = "Test City"
        self.order.zipcode = "12345"
        self.order.place = ""

        item1 = Mock()
        item1.product = Mock()
        item1.product.name = "Test Product 1"
        item1.quantity = 2
        item1.price = Money("50.00", settings.DEFAULT_CURRENCY)
        item1.total = Money("100.00", settings.DEFAULT_CURRENCY)

        item2 = Mock()
        item2.product = Mock()
        item2.product.name = "Test Product 2"
        item2.quantity = 1
        item2.price = Money("30.00", settings.DEFAULT_CURRENCY)
        item2.total = Money("30.00", settings.DEFAULT_CURRENCY)

        self.order.items = MagicMock()
        self.order.items.all.return_value = [item1, item2]

    def test_str_representation(self):
        order_str = Order.__str__(self.order)

        self.assertEqual(
            order_str,
            f"Order {self.order.id} - {self.order.first_name} {self.order.last_name}",
        )

    @patch("django.utils.timezone.now")
    def test_save_status_change(self, mock_timezone_now):
        mock_now = timezone.now()
        mock_timezone_now.return_value = mock_now

        self.order.status = OrderStatus.PROCESSING

        with patch.object(Order, "save", lambda self, *args, **kwargs: None):
            Order.save(self.order)
            self.order.status_updated_at = mock_now

        self.assertEqual(self.order.status_updated_at, mock_now)

    def test_save_status_unchanged(self):
        self.order.status = OrderStatus.PENDING
        self.order._original_status = OrderStatus.PENDING

        previous_update = timezone.now() - timedelta(days=1)
        self.order.status_updated_at = previous_update

        with patch.object(Order, "save", lambda self, *args, **kwargs: None):
            Order.save(self.order)

        self.assertEqual(self.order.status_updated_at, previous_update)

    def test_clean_valid_email(self):
        result = Order.clean(self.order)

        self.assertIsNone(result)

    @patch("order.models.order.validate_email")
    def test_clean_invalid_email(self, mock_validate_email):
        mock_validate_email.side_effect = ValidationError("Invalid email")

        with self.assertRaises(ValidationError):
            Order.clean(self.order)

    def test_total_price_items_property(self):
        expected_total = Money("130.00", settings.DEFAULT_CURRENCY)

        self.order.total_price_items = expected_total

        self.assertEqual(self.order.total_price_items, expected_total)

    def test_total_price_extra_property(self):
        expected_total = Money("10.00", settings.DEFAULT_CURRENCY)

        with patch(
            "order.models.order.settings.DEFAULT_CURRENCY",
            settings.DEFAULT_CURRENCY,
        ):
            result = Order.total_price_extra.__get__(self.order)

        self.assertEqual(result, expected_total)

    def test_full_address_property(self):
        expected_address = f"{self.order.street} {self.order.street_number}, {self.order.zipcode} {self.order.city}"

        result = Order.full_address.__get__(self.order)

        self.assertEqual(result, expected_address)

    def test_customer_full_name_property(self):
        expected_name = f"{self.order.first_name} {self.order.last_name}"

        result = Order.customer_full_name.__get__(self.order)

        self.assertEqual(result, expected_name)

    def test_is_paid_property_true(self):
        self.order.payment_status = PaymentStatus.COMPLETED

        result = Order.is_paid.__get__(self.order)

        self.assertTrue(result)

    def test_is_paid_property_false(self):
        self.order.status = OrderStatus.PENDING

        result = Order.is_paid.__get__(self.order)

        self.assertFalse(result)

    def test_can_be_canceled_property_true(self):
        self.order.status = OrderStatus.PENDING

        result = Order.can_be_canceled.__get__(self.order)

        self.assertTrue(result)

    def test_can_be_canceled_property_false(self):
        self.order.status = OrderStatus.SHIPPED

        result = Order.can_be_canceled.__get__(self.order)

        self.assertFalse(result)

    def test_is_completed_property_true(self):
        self.order.status = OrderStatus.COMPLETED

        result = Order.is_completed.__get__(self.order)

        self.assertTrue(result)

    def test_is_completed_property_false(self):
        self.order.status = OrderStatus.PENDING

        result = Order.is_completed.__get__(self.order)

        self.assertFalse(result)

    def test_is_canceled_property_true(self):
        self.order.status = OrderStatus.CANCELED

        result = Order.is_canceled.__get__(self.order)

        self.assertTrue(result)

    def test_is_canceled_property_false(self):
        self.order.status = OrderStatus.PENDING

        result = Order.is_canceled.__get__(self.order)

        self.assertFalse(result)

    def test_calculate_order_total_amount(self):
        items_total = Money("130.00", settings.DEFAULT_CURRENCY)
        shipping_total = Money("10.00", settings.DEFAULT_CURRENCY)
        expected_total = Money("140.00", settings.DEFAULT_CURRENCY)

        with patch.object(
            Order, "total_price", new_callable=PropertyMock
        ) as mock_total_price:
            mock_total_price.__get__ = Mock(return_value=expected_total)

            self.order.total_price_items = items_total
            self.order.total_price_extra = shipping_total

            self.order.total_price = expected_total

            result = Order.calculate_order_total_amount(self.order)

            self.assertEqual(result, expected_total)

    @patch("django.utils.timezone.now")
    def test_mark_as_paid(self, mock_timezone_now):
        mock_now = timezone.now()
        mock_timezone_now.return_value = mock_now

        payment_id = "PAY123"
        payment_method = "credit_card"

        original_mark_as_paid = Order.mark_as_paid

        def patched_mark_as_paid(order, **kwargs):
            original_mark_as_paid(order, **kwargs)
            order.status_updated_at = mock_now

        with (
            patch.object(Order, "mark_as_paid", patched_mark_as_paid),
            patch.object(Order, "save"),
        ):
            Order.mark_as_paid(
                self.order,
                payment_id=payment_id,
                payment_method=payment_method,
            )

        self.assertEqual(self.order.payment_status, PaymentStatus.COMPLETED)
        self.assertEqual(self.order.payment_id, payment_id)
        self.assertEqual(self.order.payment_method, payment_method)
        self.assertEqual(self.order.status_updated_at, mock_now)

    def test_add_tracking_info(self):
        tracking_number = "TRACK123"
        shipping_carrier = "FedEx"

        Order.add_tracking_info(self.order, tracking_number, shipping_carrier)

        self.assertEqual(self.order.tracking_number, tracking_number)
        self.assertEqual(self.order.shipping_carrier, shipping_carrier)


class OrderQuerySetTestCase(TestCase):
    def setUp(self):
        self.queryset = Mock(spec=OrderQuerySet)
        self.queryset.filter.return_value = self.queryset

    def test_pending_filter(self):
        result = OrderQuerySet.pending(self.queryset)
        self.queryset.filter.assert_called_once_with(status=OrderStatus.PENDING)
        self.assertEqual(result, self.queryset)

    def test_processing_filter(self):
        result = OrderQuerySet.processing(self.queryset)
        self.queryset.filter.assert_called_once_with(
            status=OrderStatus.PROCESSING
        )
        self.assertEqual(result, self.queryset)

    def test_shipped_filter(self):
        result = OrderQuerySet.shipped(self.queryset)
        self.queryset.filter.assert_called_once_with(status=OrderStatus.SHIPPED)
        self.assertEqual(result, self.queryset)

    def test_delivered_filter(self):
        result = OrderQuerySet.delivered(self.queryset)
        self.queryset.filter.assert_called_once_with(
            status=OrderStatus.DELIVERED
        )
        self.assertEqual(result, self.queryset)

    def test_completed_filter(self):
        result = OrderQuerySet.completed(self.queryset)
        self.queryset.filter.assert_called_once_with(
            status=OrderStatus.COMPLETED
        )
        self.assertEqual(result, self.queryset)

    def test_canceled_filter(self):
        result = OrderQuerySet.canceled(self.queryset)
        self.queryset.filter.assert_called_once_with(
            status=OrderStatus.CANCELED
        )
        self.assertEqual(result, self.queryset)

    def test_returned_filter(self):
        result = OrderQuerySet.returned(self.queryset)
        self.queryset.filter.assert_called_once_with(
            status=OrderStatus.RETURNED
        )
        self.assertEqual(result, self.queryset)

    def test_refunded_filter(self):
        result = OrderQuerySet.refunded(self.queryset)
        self.queryset.filter.assert_called_once_with(
            status=OrderStatus.REFUNDED
        )
        self.assertEqual(result, self.queryset)


class OrderManagerTestCase(TestCase):
    def setUp(self):
        self.manager = Mock(spec=OrderManager)
        self.queryset = Mock(spec=OrderQuerySet)
        self.manager.get_queryset.return_value = self.queryset
        self.queryset.pending.return_value = "pending_result"

    def test_pending(self):
        result = OrderManager.pending(self.manager)
        self.manager.get_queryset.assert_called_once()
        self.queryset.pending.assert_called_once()
        self.assertEqual(result, "pending_result")



from __future__ import annotations

from django.conf import settings
from drf_spectacular.utils import extend_schema_view

from tag.filters.tagged_item import TaggedItemFilter
from tag.models.tagged_item import TaggedItem
from tag.serializers.tagged_item import (
    TaggedItemDetailSerializer,
    TaggedItemSerializer,
    TaggedItemWriteSerializer,
)
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet

from core.utils.serializers import (
    create_schema_view_config,
    RequestSerializersConfig,
    ResponseSerializersConfig,
)
from core.utils.views import cache_methods

req_serializers: RequestSerializersConfig = {
    "create": TaggedItemWriteSerializer,
    "update": TaggedItemWriteSerializer,
    "partial_update": TaggedItemWriteSerializer,
}

res_serializers: ResponseSerializersConfig = {
    "create": TaggedItemDetailSerializer,
    "list": TaggedItemSerializer,
    "retrieve": TaggedItemDetailSerializer,
    "update": TaggedItemDetailSerializer,
    "partial_update": TaggedItemDetailSerializer,
}


@extend_schema_view(
    **create_schema_view_config(
        model_class=TaggedItem,
        display_config={
            "tag": "Tagged Items",
        },
        request_serializers=req_serializers,
        response_serializers=res_serializers,
        error_serializer=ErrorResponseSerializer,
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class TaggedItemViewSet(BaseModelViewSet):
    queryset = TaggedItem.objects.all()
    response_serializers = res_serializers
    request_serializers = req_serializers
    filterset_class = TaggedItemFilter
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "object_id",
        "tag__translations__label",
    ]
    ordering = ["-created_at"]
    search_fields = ["tag__translations__label", "content_type__model"]



from django.contrib import admin
from django.db import models
from django.db.models import Q
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from parler.admin import TranslatableAdmin
from unfold.admin import ModelAdmin
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateTimeFilter,
    RangeNumericListFilter,
)
from unfold.contrib.forms.widgets import WysiwygWidget
from unfold.decorators import action
from unfold.enums import ActionVariant

from pay_way.models import PayWay


class CostRangeFilter(RangeNumericListFilter):
    title = _("Cost Range")
    parameter_name = "cost_range"

    def queryset(self, request, queryset):
        filters = {}
        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["cost__gte"] = value_from
        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["cost__lte"] = value_to
        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


class FreeThresholdFilter(RangeNumericListFilter):
    title = _("Free Threshold Range")
    parameter_name = "free_threshold_range"

    def queryset(self, request, queryset):
        filters = {}
        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["free_threshold__gte"] = value_from
        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["free_threshold__lte"] = value_to
        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


class PaymentTypeFilter(DropdownFilter):
    title = _("Payment Type")
    parameter_name = "payment_type"

    def lookups(self, request, model_admin):
        return [
            ("online", _("Online Payment")),
            ("offline_simple", _("Offline (Simple)")),
            ("offline_confirmation", _("Offline (Requires Confirmation)")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "online":
            return queryset.filter(is_online_payment=True)
        elif self.value() == "offline_simple":
            return queryset.filter(
                is_online_payment=False, requires_confirmation=False
            )
        elif self.value() == "offline_confirmation":
            return queryset.filter(
                is_online_payment=False, requires_confirmation=True
            )
        return queryset


class ConfigurationStatusFilter(DropdownFilter):
    title = _("Configuration Status")
    parameter_name = "configuration_status"

    def lookups(self, request, model_admin):
        return [
            ("configured", _("Configured")),
            ("not_configured", _("Not Configured")),
            ("no_config_needed", _("No Configuration Needed")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "configured":
            return queryset.filter(
                is_online_payment=True, configuration__isnull=False
            ).exclude(configuration={})
        elif self.value() == "not_configured":
            return queryset.filter(is_online_payment=True).filter(
                Q(configuration__isnull=True) | Q(configuration={})
            )
        elif self.value() == "no_config_needed":
            return queryset.filter(is_online_payment=False)
        return queryset


@admin.register(PayWay)
class PayWayAdmin(ModelAdmin, TranslatableAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    formfield_overrides = {
        models.TextField: {"widget": WysiwygWidget},
    }

    list_display = [
        "name_display",
        "provider_code_badge",
        "active_status",
        "active",
        "payment_type_display",
        "cost_display",
        "free_threshold_display",
        "configuration_status",
        "icon_preview",
        "sort_order_display",
    ]

    list_filter = [
        "active",
        PaymentTypeFilter,
        ConfigurationStatusFilter,
        "is_online_payment",
        "requires_confirmation",
        CostRangeFilter,
        FreeThresholdFilter,
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
    ]

    list_editable = ["active"]

    search_fields = [
        "translations__name",
        "provider_code",
        "translations__description",
        "translations__instructions",
    ]

    readonly_fields = [
        "id",
        "created_at",
        "updated_at",
        "configuration_preview",
        "effective_cost_display",
        "is_configured_status",
        "sort_order",
    ]

    ordering = ["sort_order", "id"]

    actions = [
        "activate_payment_methods",
        "deactivate_payment_methods",
        "move_up_in_order",
        "move_down_in_order",
        "reset_sort_order",
    ]

    fieldsets = (
        (
            _("Basic Information"),
            {"fields": ("active", "sort_order"), "classes": ("wide",)},
        ),
        (
            _("Display & Branding"),
            {"fields": ("name", "icon"), "classes": ("wide",)},
        ),
        (
            _("Content"),
            {"fields": ("description", "instructions"), "classes": ("wide",)},
        ),
        (
            _("Payment Configuration"),
            {
                "fields": (
                    "provider_code",
                    "is_online_payment",
                    "requires_confirmation",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Pricing"),
            {
                "fields": ("cost", "free_threshold", "effective_cost_display"),
                "classes": ("wide",),
            },
        ),
        (
            _("Advanced Configuration"),
            {
                "fields": (
                    "configuration",
                    "configuration_preview",
                    "is_configured_status",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def name_display(self, obj):
        name = (
            obj.safe_translation_getter("name", any_language=True)
            or "Unnamed Payment Method"
        )
        safe_name = conditional_escape(name)
        html = f'<strong class="text-base-900 dark:text-base-100">{safe_name}</strong>'
        return mark_safe(html)

    name_display.short_description = _("Name")

    def provider_code_badge(self, obj):
        if obj.provider_code:
            safe_code = conditional_escape(obj.provider_code)
            html = (
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f'bg-indigo-50 dark:bg-indigo-900 text-indigo-700 dark:text-indigo-300 rounded-full">'
                f"{safe_code}"
                f"</span>"
            )
            return mark_safe(html)
        return mark_safe(
            '<span class="text-base-400 dark:text-base-500 italic">'
            "No provider"
            "</span>"
        )

    provider_code_badge.short_description = _("Provider")

    def active_status(self, obj):
        if obj.active:
            return mark_safe(
                '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
                'bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full gap-1">'
                "<span>✓</span><span>Active</span>"
                "</span>"
            )
        return mark_safe(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
            'bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full gap-1">'
            "<span>✗</span><span>Inactive</span>"
            "</span>"
        )

    active_status.short_description = _("Status")

    def payment_type_display(self, obj):
        if obj.is_online_payment:
            return mark_safe(
                '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
                'bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full gap-1">'
                "<span>🌐</span><span>Online</span>"
                "</span>"
            )
        elif obj.requires_confirmation:
            return mark_safe(
                '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
                'bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-300 rounded-full gap-1">'
                "<span>⏱️</span><span>Offline (Confirm)</span>"
                "</span>"
            )
        return mark_safe(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
            'bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-300 rounded-full gap-1">'
            "<span>📝</span><span>Offline</span>"
            "</span>"
        )

    payment_type_display.short_description = _("Type")

    def cost_display(self, obj):
        if obj.cost and obj.cost.amount > 0:
            safe_amount = conditional_escape(str(obj.cost.amount))
            safe_currency = conditional_escape(obj.cost.currency)
            html = (
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f'bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300 rounded-full">'
                f"{safe_amount} {safe_currency}"
                f"</span>"
            )
            return mark_safe(html)
        return mark_safe(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
            'bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full">'
            "Free"
            "</span>"
        )

    cost_display.short_description = _("Cost")

    def free_threshold_display(self, obj):
        if obj.free_threshold and obj.free_threshold.amount > 0:
            safe_amount = conditional_escape(str(obj.free_threshold.amount))
            safe_currency = conditional_escape(obj.free_threshold.currency)
            html = (
                f'<span class="text-sm text-base-700 dark:text-base-300">'
                f"Free above {safe_amount} {safe_currency}"
                f"</span>"
            )
            return mark_safe(html)
        return mark_safe(
            '<span class="text-base-400 dark:text-base-500">No threshold</span>'
        )

    free_threshold_display.short_description = _("Free Threshold")

    def configuration_status(self, obj):
        if not obj.is_online_payment:
            return mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                'bg-base-50 dark:bg-base-900 text-base-700 dark:text-base-300 rounded-full">'
                "N/A"
                "</span>"
            )
        if obj.is_configured:
            return mark_safe(
                '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
                'bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full gap-1">'
                "<span>✓</span><span>Configured</span>"
                "</span>"
            )
        return mark_safe(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
            'bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full gap-1">'
            "<span>⚠️</span><span>Missing Config</span>"
            "</span>"
        )

    configuration_status.short_description = _("Configuration")

    def sort_order_display(self, obj):
        if obj.sort_order is not None:
            safe_order = conditional_escape(str(obj.sort_order))
            html = (
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f'bg-base-50 dark:bg-base-900 text-base-700 dark:text-base-300 rounded-full">'
                f"#{safe_order}"
                f"</span>"
            )
            return mark_safe(html)
        return mark_safe(
            '<span class="text-base-400 dark:text-base-500">-</span>'
        )

    sort_order_display.short_description = _("Order")

    def icon_preview(self, obj):
        if obj.icon:
            safe_url = conditional_escape(obj.icon.url)
            html = (
                f'<img src="{safe_url}" style="max-height: 32px; max-width: 64px; '
                'border-radius: 4px; object-fit: contain;" />'
            )
            return mark_safe(html)
        return mark_safe(
            '<span class="text-base-400 dark:text-base-500">No icon</span>'
        )

    icon_preview.short_description = _("Icon")

    def configuration_preview(self, obj):
        if not obj.configuration:
            return mark_safe(
                '<span class="text-base-400 dark:text-base-500 italic">No configuration</span>'
            )
        keys = list(obj.configuration.keys())
        if len(keys) > 3:
            display = keys[:3] + [f"... and {len(keys) - 3} more"]
        else:
            display = keys
        safe_list = conditional_escape(", ".join(display))
        html = (
            '<div class="text-sm">'
            '<div class="font-medium text-base-700 dark:text-base-300">Configuration Keys:</div>'
            f'<div class="text-base-500 dark:text-base-400">{safe_list}</div>'
            "</div>"
        )
        return mark_safe(html)

    configuration_preview.short_description = _("Configuration Preview")

    def effective_cost_display(self, obj):
        if obj.cost:
            formatted = f"{obj.effective_cost} {obj.cost.currency}"
        else:
            formatted = "0"
        safe_fmt = conditional_escape(formatted)
        html = (
            f'<span class="text-sm font-medium text-base-700 dark:text-base-300">'
            f"{safe_fmt}"
            "</span>"
        )
        return mark_safe(html)

    effective_cost_display.short_description = _("Effective Cost")

    def is_configured_status(self, obj):
        if obj.is_configured:
            return mark_safe(
                '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
                'bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full gap-1">'
                "<span>✓</span><span>Ready to use</span>"
                "</span>"
            )
        return mark_safe(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
            'bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full gap-1">'
            "<span>⚠️</span><span>Requires setup</span>"
            "</span>"
        )

    is_configured_status.short_description = _("Ready Status")

    @action(
        description=_("Activate selected payment methods"),
        variant=ActionVariant.SUCCESS,
        icon="check_circle",
    )
    def activate_payment_methods(self, request, queryset):
        updated = queryset.update(active=True)
        self.message_user(
            request,
            _("%(count)d payment methods were successfully activated.")
            % {"count": updated},
        )

    @action(
        description=_("Deactivate selected payment methods"),
        variant=ActionVariant.WARNING,
        icon="cancel",
    )
    def deactivate_payment_methods(self, request, queryset):
        updated = queryset.update(active=False)
        self.message_user(
            request,
            _("%(count)d payment methods were successfully deactivated.")
            % {"count": updated},
        )

    @action(
        description=_("Move selected items up in sort order"),
        variant=ActionVariant.INFO,
        icon="keyboard_arrow_up",
    )
    def move_up_in_order(self, request, queryset):
        moved_count = 0
        for obj in queryset.order_by("sort_order"):
            if obj.sort_order and obj.sort_order > 0:
                obj.move_up()
                moved_count += 1
        self.message_user(
            request,
            _("%(count)d payment methods moved up in sort order.")
            % {"count": moved_count},
        )

    @action(
        description=_("Move selected items down in sort order"),
        variant=ActionVariant.INFO,
        icon="keyboard_arrow_down",
    )
    def move_down_in_order(self, request, queryset):
        moved_count = 0
        for obj in queryset.order_by("-sort_order"):
            obj.move_down()
            moved_count += 1
        self.message_user(
            request,
            _("%(count)d payment methods moved down in sort order.")
            % {"count": moved_count},
        )

    @action(
        description=_("Reset sort order to default"),
        variant=ActionVariant.INFO,
        icon="sort",
    )
    def reset_sort_order(self, request, queryset):
        for index, obj in enumerate(queryset.order_by("id"), start=1):
            obj.sort_order = index * 10
            obj.save(update_fields=["sort_order"])
        self.message_user(
            request,
            _("Sort order has been reset for %(count)d payment methods.")
            % {"count": queryset.count()},
        )



# Generated by Django 5.2.1 on 2025-05-13 10:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0007_alter_notification_link'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='kind',
            field=models.CharField(choices=[('ERROR', 'Error'), ('SUCCESS', 'Success'), ('INFO', 'Info'), ('WARNING', 'Warning'), ('DANGER', 'Danger')], default='INFO', max_length=250, verbose_name='Kind'),
        ),
    ]



from __future__ import annotations

from django.conf import settings
from django.db.models import Count
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import (
    extend_schema,
    extend_schema_view,
)
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response

from blog.filters.comment import BlogCommentFilter
from blog.models.comment import BlogComment
from blog.serializers.comment import (
    BlogCommentDetailSerializer,
    BlogCommentLikedCommentsRequestSerializer,
    BlogCommentLikedCommentsResponseSerializer,
    BlogCommentSerializer,
    BlogCommentWriteSerializer,
)
from blog.serializers.post import (
    BlogPostDetailSerializer,
)
from core.api.permissions import IsOwnerOrAdmin
from core.api.serializers import ErrorResponseSerializer
from core.api.views import BaseModelViewSet

from core.utils.serializers import (
    create_schema_view_config,
    RequestSerializersConfig,
    ResponseSerializersConfig,
)
from core.utils.views import cache_methods

req_serializers: RequestSerializersConfig = {
    "create": BlogCommentWriteSerializer,
    "update": BlogCommentWriteSerializer,
    "partial_update": BlogCommentWriteSerializer,
    "liked_comments": BlogCommentLikedCommentsRequestSerializer,
}

res_serializers: ResponseSerializersConfig = {
    "create": BlogCommentDetailSerializer,
    "list": BlogCommentSerializer,
    "retrieve": BlogCommentDetailSerializer,
    "update": BlogCommentDetailSerializer,
    "partial_update": BlogCommentDetailSerializer,
    "replies": BlogCommentSerializer,
    "thread": BlogCommentSerializer,
    "update_likes": BlogCommentDetailSerializer,
    "post": BlogPostDetailSerializer,
    "liked_comments": BlogCommentLikedCommentsResponseSerializer,
    "my_comments": BlogCommentSerializer,
}


@extend_schema_view(
    **create_schema_view_config(
        model_class=BlogComment,
        display_config={
            "tag": "Blog Comments",
        },
        request_serializers=req_serializers,
        response_serializers=res_serializers,
        error_serializer=ErrorResponseSerializer,
    )
)
@cache_methods(settings.DEFAULT_CACHE_TTL, methods=["list", "retrieve"])
class BlogCommentViewSet(BaseModelViewSet):
    queryset = BlogComment.objects.all()
    request_serializers = req_serializers
    response_serializers = res_serializers

    filterset_class = BlogCommentFilter
    ordering_fields = [
        "id",
        "created_at",
        "updated_at",
        "level",
        "lft",
        "approved",
    ]
    ordering = ["-created_at"]
    search_fields = [
        "translations__content",
        "user__email",
        "user__first_name",
        "user__last_name",
        "post__translations__title",
    ]

    def get_queryset(self):
        queryset = BlogComment.objects.select_related(
            "user", "post", "parent", "post__category", "post__author"
        ).prefetch_related(
            "likes",
            "translations",
            "children",
            "post__translations",
        )

        if not self.request.user.is_staff:
            queryset = queryset.filter(approved=True)

        return queryset

    def get_permissions(self):
        permission_classes = []
        if self.action in [
            "create",
            "update",
            "partial_update",
            "destroy",
            "update_likes",
            "liked_comments",
            "my_comments",
        ]:
            permission_classes.append(IsOwnerOrAdmin)
        return [permission() for permission in permission_classes]

    @extend_schema(
        operation_id="listBlogCommentReplies",
        summary=_("Get comment replies"),
        description=_(
            "Get all replies (children) of this comment in threaded structure."
        ),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentSerializer(many=True),
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def replies(self, request, pk=None):
        comment = self.get_object()
        queryset = (
            comment.get_children()
            .select_related("user", "post")
            .prefetch_related("likes")
            .filter(approved=True)
            .order_by("created_at")
        )

        response_serializer_class = self.get_response_serializer()
        return self.paginate_and_serialize(
            queryset, request, serializer_class=response_serializer_class
        )

    @extend_schema(
        operation_id="getBlogCommentThread",
        summary=_("Get comment thread"),
        description=_(
            "Get the complete thread (all ancestors and descendants) of this comment."
        ),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentSerializer(many=True),
        },
    )
    @action(detail=True, methods=["GET"])
    def thread(self, request, pk=None):
        comment = self.get_object()

        if self.request.user.is_staff:
            ancestors = comment.get_ancestors().select_related("user", "post")
            descendants = comment.get_descendants().select_related(
                "user", "post"
            )
        else:
            ancestors = (
                comment.get_ancestors()
                .select_related("user", "post")
                .filter(approved=True)
            )
            descendants = (
                comment.get_descendants()
                .select_related("user", "post")
                .filter(approved=True)
            )

        queryset = [*list(ancestors), comment, *list(descendants)]
        queryset = sorted(queryset, key=lambda x: x.created_at)

        response_serializer_class = self.get_response_serializer()
        return self.paginate_and_serialize(
            queryset, request, serializer_class=response_serializer_class
        )

    @extend_schema(
        operation_id="toggleBlogCommentLike",
        summary=_("Toggle comment like"),
        description=_("Like or unlike a comment. Toggles the like status."),
        tags=["Blog Comments"],
        request=None,
        responses={
            200: BlogCommentDetailSerializer,
            401: ErrorResponseSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["POST"])
    def update_likes(self, request, pk=None):
        comment = self.get_object()
        user = request.user

        if comment.likes.filter(id=user.id).exists():
            comment.likes.remove(user)
            liked = False
        else:
            comment.likes.add(user)
            liked = True

        comment.refresh_from_db()

        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(
            comment, context=self.get_serializer_context()
        )
        data = response_serializer.data.copy()
        data["action"] = "liked" if liked else "unliked"

        return Response(data, status=status.HTTP_200_OK)

    @extend_schema(
        operation_id="getBlogCommentPost",
        summary=_("Get comment's blog post"),
        description=_("Get the blog post that this comment belongs to."),
        tags=["Blog Comments"],
        responses={
            200: BlogPostDetailSerializer,
            404: ErrorResponseSerializer,
        },
    )
    @action(detail=True, methods=["GET"])
    def post(self, request, pk=None):
        self.ordering_fields = []
        self.ordering = []
        self.search_fields = []

        comment = self.get_object()
        post = comment.post

        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(
            post, context=self.get_serializer_context()
        )
        return Response(response_serializer.data)

    @extend_schema(
        operation_id="checkBlogCommentLikes",
        summary=_("Check bulk like status"),
        description=_(
            "Check which comments from a list are liked by the current user."
        ),
        tags=["Blog Comments"],
        request=BlogCommentLikedCommentsRequestSerializer,
        responses={
            200: BlogCommentLikedCommentsResponseSerializer,
            400: ErrorResponseSerializer,
            401: ErrorResponseSerializer,
        },
    )
    @action(detail=False, methods=["POST"])
    def liked_comments(self, request, *args, **kwargs):
        request_serializer_class = self.get_request_serializer()
        serializer = request_serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        comment_ids = serializer.validated_data["comment_ids"]
        user = request.user

        queryset = BlogComment.objects.filter(id__in=comment_ids, likes=user)
        if not self.request.user.is_staff:
            queryset = queryset.filter(approved=True)

        liked_ids = list(queryset.values_list("id", flat=True))

        response_data = {"liked_comment_ids": liked_ids}

        response_serializer_class = self.get_response_serializer()
        response_serializer = response_serializer_class(response_data)
        return Response(response_serializer.data)

    @extend_schema(
        operation_id="listMyBlogComments",
        summary=_("Get current user's comments"),
        description=_(
            "Get all comments made by the currently authenticated user."
        ),
        tags=["Blog Comments"],
        responses={
            200: BlogCommentSerializer(many=True),
            401: ErrorResponseSerializer,
        },
    )
    @action(detail=False, methods=["GET"])
    def my_comments(self, request):
        queryset = self.get_queryset().filter(user=request.user)

        if (
            hasattr(queryset.query.where, "children")
            and queryset.query.where.children
        ):
            queryset = (
                BlogComment.objects.select_related(
                    "user", "post", "parent", "post__category", "post__author"
                )
                .prefetch_related(
                    "likes",
                    "translations",
                    "children",
                    "post__translations",
                )
                .annotate(
                    likes_count_field=Count("likes", distinct=True),
                )
                .filter(user=request.user)
            )

        response_serializer_class = self.get_response_serializer()
        return self.paginate_and_serialize(
            queryset, request, serializer_class=response_serializer_class
        )



import factory
from django.contrib.contenttypes.models import ContentType

from core.helpers.factory import get_or_create_instance
from tag.models.tagged_item import TaggedItem


def get_or_create_product():
    return get_or_create_instance(
        app_label="product",
        model_name="Product",
        factory_module_path="product.factories.product",
        factory_class_name="ProductFactory",
    )


def get_or_create_tag():
    return get_or_create_instance(
        app_label="tag",
        model_name="Tag",
        factory_module_path="tag.factories.tag",
        factory_class_name="TagFactory",
    )


class TaggedItemFactory(factory.django.DjangoModelFactory):
    tag = factory.LazyFunction(get_or_create_tag)

    class Meta:
        abstract = True

    @classmethod
    def _create(cls, model_class, **kwargs):
        content_object = kwargs.pop("content_object", None)

        if content_object is None:
            raise ValueError("content_object is required for TaggedItem")

        if hasattr(content_object, "pk") and content_object.pk is None:
            content_object.save()

        content_type = ContentType.objects.get_for_model(content_object)
        content_type = ContentType.objects.get(pk=content_type.pk)

        kwargs.update(
            {
                "content_type": content_type,
                "object_id": content_object.pk,
            }
        )

        return super()._create(model_class, **kwargs)


class TaggedProductFactory(TaggedItemFactory):
    content_object = factory.LazyFunction(get_or_create_product)

    class Meta:
        model = TaggedItem



from django.urls import path
from rest_framework.urlpatterns import format_suffix_patterns

from user.views.account import UserAccountViewSet
from user.views.address import UserAddressViewSet
from user.views.subscription import (
    SubscriptionTopicViewSet,
    UnsubscribeView,
    UserSubscriptionViewSet,
)

urlpatterns = [
    path(
        "user/account",
        UserAccountViewSet.as_view({"get": "list", "post": "create"}),
        name="user-account-list",
    ),
    path(
        "user/account/<int:pk>",
        UserAccountViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="user-account-detail",
    ),
    path(
        "user/account/<int:pk>/favourite_products",
        UserAccountViewSet.as_view({"get": "favourite_products"}),
        name="user-account-favourite-products",
    ),
    path(
        "user/account/<int:pk>/orders",
        UserAccountViewSet.as_view({"get": "orders"}),
        name="user-account-orders",
    ),
    path(
        "user/account/<int:pk>/product_reviews",
        UserAccountViewSet.as_view({"get": "product_reviews"}),
        name="user-account-product-reviews",
    ),
    path(
        "user/account/<int:pk>/addresses",
        UserAccountViewSet.as_view({"get": "addresses"}),
        name="user-account-addresses",
    ),
    path(
        "user/account/<int:pk>/blog_post_comments",
        UserAccountViewSet.as_view({"get": "blog_post_comments"}),
        name="user-account-blog-post-comments",
    ),
    path(
        "user/account/<int:pk>/liked_blog_posts",
        UserAccountViewSet.as_view({"get": "liked_blog_posts"}),
        name="user-account-liked-blog-posts",
    ),
    path(
        "user/account/<int:pk>/liked_blog_posts",
        UserAccountViewSet.as_view({"get": "liked_blog_posts"}),
        name="user-account-liked-blog-posts",
    ),
    path(
        "user/account/<int:pk>/notifications",
        UserAccountViewSet.as_view({"get": "notifications"}),
        name="user-account-notifications",
    ),
    path(
        "user/account/<int:pk>/change_username",
        UserAccountViewSet.as_view({"post": "change_username"}),
        name="user-account-change-username",
    ),
    path(
        "user/address",
        UserAddressViewSet.as_view({"get": "list", "post": "create"}),
        name="user-address-list",
    ),
    path(
        "user/address/<int:pk>",
        UserAddressViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="user-address-detail",
    ),
    path(
        "user/address/<int:pk>/set_main",
        UserAddressViewSet.as_view({"post": "set_main"}),
        name="user-address-set-main",
    ),
    path(
        "user/subscription",
        UserSubscriptionViewSet.as_view({"get": "list", "post": "create"}),
        name="user-subscription-list",
    ),
    path(
        "user/subscription/<int:pk>",
        UserSubscriptionViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="user-subscription-detail",
    ),
    path(
        "user/subscription/bulk_update",
        UserSubscriptionViewSet.as_view({"post": "bulk_update"}),
        name="user-subscription-bulk-update",
    ),
    path(
        "user/subscription/<int:pk>/confirm",
        UserSubscriptionViewSet.as_view({"post": "confirm"}),
        name="user-subscription-confirm",
    ),
    path(
        "user/subscription/topic",
        SubscriptionTopicViewSet.as_view({"get": "list", "post": "create"}),
        name="user-subscription-topic-list",
    ),
    path(
        "user/subscription/topic/my_subscriptions",
        SubscriptionTopicViewSet.as_view({"get": "my_subscriptions"}),
        name="user-subscription-topic-my-subscriptions",
    ),
    path(
        "user/subscription/topic/<int:pk>",
        SubscriptionTopicViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="user-subscription-topic-detail",
    ),
    path(
        "user/subscription/topic/<int:pk>/subscribe",
        SubscriptionTopicViewSet.as_view({"post": "subscribe"}),
        name="user-subscription-topic-subscribe",
    ),
    path(
        "user/subscription/topic/<int:pk>/unsubscribe",
        SubscriptionTopicViewSet.as_view({"post": "unsubscribe"}),
        name="user-subscription-topic-unsubscribe",
    ),
    path(
        "user/unsubscribe/<str:uidb64>/<str:token>/<str:topic_slug>",
        UnsubscribeView.as_view(),
        name="user-unsubscribe",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



from unittest.mock import MagicMock, patch

import pytest
from allauth.headless.tokens.sessions import (
    SessionTokenStrategy as BaseSessionTokenStrategy,
)
from django.contrib.auth import get_user_model
from django.test import RequestFactory
from knox.models import get_token_model

from core.api.tokens import AuthToken, SessionTokenStrategy

User = get_user_model()


class TestSessionTokenStrategy:
    def setup_method(self):
        self.strategy = SessionTokenStrategy()
        self.factory = RequestFactory()

    def test_strategy_inherits_from_base_session_token_strategy(self):
        assert isinstance(self.strategy, BaseSessionTokenStrategy)

    @patch("core.api.tokens.AuthToken.objects.create")
    def test_create_access_token_success(self, mock_create):
        user = User(id=1, email="test@example.com")
        request = self.factory.get("/")
        request.user = user

        mock_token_instance = MagicMock()
        mock_token_value = "test_token_value_123"
        mock_create.return_value = (mock_token_instance, mock_token_value)

        token = self.strategy.create_access_token(request)

        assert token == mock_token_value
        mock_create.assert_called_once_with(user)

    @patch("core.api.tokens.AuthToken.objects.create")
    def test_create_access_token_with_different_users(self, mock_create):
        user1 = User(id=1, email="user1@example.com")
        request1 = self.factory.get("/")
        request1.user = user1

        mock_create.return_value = (MagicMock(), "token1")
        token1 = self.strategy.create_access_token(request1)
        assert token1 == "token1"

        user2 = User(id=2, email="user2@example.com")
        request2 = self.factory.get("/")
        request2.user = user2

        mock_create.return_value = (MagicMock(), "token2")
        token2 = self.strategy.create_access_token(request2)
        assert token2 == "token2"

        assert mock_create.call_count == 2

    @patch("core.api.tokens.AuthToken.objects.create")
    def test_create_access_token_exception_handling(self, mock_create):
        user = User(id=1, email="testuser@example.com")
        request = self.factory.get("/")
        request.user = user

        mock_create.side_effect = Exception("Token creation failed")

        with pytest.raises(Exception, match="Token creation failed"):
            self.strategy.create_access_token(request)

    def test_auth_token_model_import(self):
        expected_model = get_token_model()
        assert AuthToken == expected_model

    def test_strategy_has_required_methods(self):
        assert hasattr(self.strategy, "create_access_token")
        assert callable(self.strategy.create_access_token)

    @patch("core.api.tokens.AuthToken.objects.create")
    def test_create_access_token_return_type(self, mock_create):
        user = User(id=1, email="testuser@example.com")
        request = self.factory.get("/")
        request.user = user

        mock_instance = MagicMock()
        mock_token = "expected_token_string"
        mock_create.return_value = (mock_instance, mock_token)

        result = self.strategy.create_access_token(request)

        assert result == mock_token
        assert isinstance(result, str)

    def test_strategy_class_instantiation(self):
        strategy = SessionTokenStrategy()
        assert strategy is not None
        assert hasattr(strategy, "create_access_token")


class TestAuthTokenModel:
    def test_auth_token_model_is_imported(self):
        assert AuthToken is not None
        assert hasattr(AuthToken, "objects")

    def test_auth_token_get_token_model(self):
        token_model = get_token_model()
        assert AuthToken == token_model

    def test_auth_token_has_manager(self):
        assert hasattr(AuthToken, "objects")
        assert hasattr(AuthToken.objects, "create")



from unittest.mock import patch

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.test import RequestFactory
from rest_framework.throttling import UserRateThrottle

from core.api.throttling import BurstRateThrottle

User = get_user_model()


class TestBurstRateThrottle:
    def setup_method(self):
        self.throttle = BurstRateThrottle()
        self.factory = RequestFactory()

    def test_throttle_scope_configuration(self):
        assert self.throttle.scope == "burst"

    def test_throttle_inherits_from_user_rate_throttle(self):
        assert isinstance(self.throttle, UserRateThrottle)

    def test_throttle_rate_configuration(self):
        request = self.factory.get("/")
        request.user = User()

        rate = self.throttle.get_rate()
        assert rate is None or isinstance(rate, str)

    def test_throttle_cache_key_generation(self):
        request = self.factory.get("/")
        request.user = User()
        request.user.pk = 123

        cache_key = self.throttle.get_cache_key(request, view=None)

        assert cache_key is not None
        assert "burst" in cache_key.lower() or "123" in str(cache_key)

    def test_throttle_anonymous_user(self):
        request = self.factory.get("/")
        request.user = AnonymousUser()

        cache_key = self.throttle.get_cache_key(request, view=None)
        assert cache_key is not None
        assert "burst" in cache_key

    @patch.dict(
        settings.REST_FRAMEWORK, {"DEFAULT_THROTTLE_RATES": {"burst": "2/min"}}
    )
    def test_throttle_allow_request(self):
        request = self.factory.get("/")
        request.user = User()
        request.user.pk = 456

        with patch.object(self.throttle, "cache") as mock_cache:
            mock_cache.get.return_value = None
            mock_cache.set.return_value = None

            result = self.throttle.allow_request(request, view=None)
            assert result is True

    def test_throttle_get_ident(self):
        request = self.factory.get("/")
        request.user = User()
        request.user.pk = 789

        ident = self.throttle.get_ident(request)

        assert ident is not None
        assert isinstance(ident, str)

    def test_throttle_scope_attribute_exists(self):
        assert hasattr(BurstRateThrottle, "scope")
        assert BurstRateThrottle.scope == "burst"

    def test_throttle_class_instantiation(self):
        throttle_instance = BurstRateThrottle()
        assert throttle_instance.scope == "burst"
        assert hasattr(throttle_instance, "allow_request")
        assert hasattr(throttle_instance, "get_cache_key")
        assert hasattr(throttle_instance, "get_rate")



from rest_framework import pagination
from rest_framework.response import Response


class PageNumberPaginator(pagination.PageNumberPagination):
    page_size_query_param = "page_size"
    max_page_size = 100

    def get_paginated_response(self, data):
        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.page.paginator.count,
                "total_pages": self.page.paginator.num_pages,
                "page_size": self.page.paginator.per_page,
                "page_total_results": len(data),
                "page": self.page.number,
                "results": data,
            }
        )

    def get_paginated_response_schema(self, schema):
        return {
            "type": "object",
            "required": ["count", "results"],
            "properties": {
                "links": {
                    "type": "object",
                    "properties": {
                        "next": {
                            "type": "string",
                            "nullable": True,
                            "format": "uri",
                            "example": "http://api.example.org/accounts/?{page_query_param}=4".format(
                                page_query_param=self.page_query_param
                            ),
                        },
                        "previous": {
                            "type": "string",
                            "nullable": True,
                            "format": "uri",
                            "example": "http://api.example.org/accounts/?{page_query_param}=2".format(
                                page_query_param=self.page_query_param
                            ),
                        },
                    },
                },
                "count": {
                    "type": "integer",
                    "example": 123,
                },
                "total_pages": {
                    "type": "integer",
                    "example": 123,
                },
                "page_size": {
                    "type": "integer",
                    "example": 123,
                },
                "page_total_results": {
                    "type": "integer",
                    "example": 123,
                },
                "page": {
                    "type": "integer",
                    "example": 123,
                },
                "results": schema,
            },
        }



from django.utils.translation import gettext_lazy as _
from djmoney.contrib.django_rest_framework import MoneyField
from drf_spectacular.utils import extend_schema_field
from phonenumber_field.serializerfields import PhoneNumberField
from rest_framework import serializers
from rest_framework.relations import PrimaryKeyRelatedField

from core.utils.email import is_disposable_domain
from country.models import Country
from order.enum.status import OrderStatus
from order.models.history import OrderHistory
from order.models.item import OrderItem
from order.models.order import Order
from order.serializers.item import (
    OrderItemCreateSerializer,
    OrderItemSerializer,
    OrderItemDetailSerializer,
)
from order.signals import order_created
from pay_way.models import PayWay
from product.models.product import Product
from region.models import Region


class OrderSerializer(serializers.ModelSerializer[Order]):
    items = OrderItemSerializer(many=True)
    country = PrimaryKeyRelatedField(queryset=Country.objects.all())
    region = PrimaryKeyRelatedField(queryset=Region.objects.all())
    pay_way = PrimaryKeyRelatedField(queryset=PayWay.objects.all())
    paid_amount = MoneyField(max_digits=11, decimal_places=2, read_only=True)
    shipping_price = MoneyField(max_digits=11, decimal_places=2, read_only=True)
    total_price_items = MoneyField(
        max_digits=11, decimal_places=2, read_only=True
    )
    total_price_extra = MoneyField(
        max_digits=11, decimal_places=2, read_only=True
    )
    phone = PhoneNumberField()
    mobile_phone = PhoneNumberField(required=False, allow_blank=True)
    status_display = serializers.SerializerMethodField("get_status_display")
    can_be_canceled = serializers.BooleanField(read_only=True)
    is_paid = serializers.BooleanField(read_only=True)

    def get_status_display(self, order: Order) -> str:
        return order.get_status_display()

    class Meta:
        model = Order
        fields = (
            "id",
            "user",
            "country",
            "region",
            "floor",
            "location_type",
            "street",
            "street_number",
            "pay_way",
            "status",
            "status_display",
            "status_updated_at",
            "first_name",
            "last_name",
            "email",
            "zipcode",
            "place",
            "city",
            "phone",
            "mobile_phone",
            "customer_notes",
            "paid_amount",
            "items",
            "shipping_price",
            "document_type",
            "created_at",
            "updated_at",
            "uuid",
            "total_price_items",
            "total_price_extra",
            "full_address",
            "payment_id",
            "payment_status",
            "payment_method",
            "can_be_canceled",
            "is_paid",
        )
        read_only_fields = (
            "id",
            "uuid",
            "paid_amount",
            "shipping_price",
            "total_price_items",
            "total_price_extra",
            "created_at",
            "updated_at",
            "status_updated_at",
            "can_be_canceled",
            "is_paid",
        )


class OrderDetailSerializer(OrderSerializer):
    items = OrderItemDetailSerializer(many=True)
    order_timeline = serializers.SerializerMethodField(
        help_text="Order status timeline and history"
    )
    pricing_breakdown = serializers.SerializerMethodField(
        help_text="Detailed pricing breakdown"
    )
    tracking_details = serializers.SerializerMethodField(
        help_text="Tracking and shipping details"
    )
    phone = PhoneNumberField(read_only=True)
    mobile_phone = PhoneNumberField(read_only=True, allow_blank=True)

    @extend_schema_field(
        {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "change_type": {"type": "string"},
                    "timestamp": {"type": "string"},
                    "description": {"type": "string"},
                    "user": {"type": "string", "nullable": True},
                    "previous_value": {"type": "object", "nullable": True},
                    "new_value": {"type": "object", "nullable": True},
                },
            },
        }
    )
    def get_order_timeline(self, obj):
        timeline = []

        timeline.append(
            {
                "change_type": "CREATED",
                "timestamp": obj.created_at,
                "description": _("Order was created"),
                "user": None,
                "previous_value": None,
                "new_value": None,
            }
        )

        history_records = OrderHistory.objects.filter(order=obj).order_by(
            "created_at"
        )

        for history in history_records:
            timeline.append(
                {
                    "change_type": history.change_type,
                    "timestamp": history.created_at,
                    "description": history.description,
                    "user": history.user.full_name if history.user else None,
                    "previous_value": history.previous_value,
                    "new_value": history.new_value,
                }
            )

        return timeline

    @extend_schema_field(
        {
            "type": "object",
            "properties": {
                "items_subtotal": {"type": "number"},
                "shipping_cost": {"type": "number"},
                "extras_total": {"type": "number"},
                "grand_total": {"type": "number"},
                "currency": {"type": "string"},
                "paid_amount": {"type": "number"},
                "remaining_amount": {"type": "number"},
            },
        }
    )
    def get_pricing_breakdown(self, obj) -> dict:
        items_total = (
            obj.total_price_items.amount if obj.total_price_items else 0
        )
        shipping_total = obj.shipping_price.amount if obj.shipping_price else 0
        extras_total = (
            obj.total_price_extra.amount if obj.total_price_extra else 0
        )

        grand_total = items_total + shipping_total

        return {
            "items_subtotal": items_total,
            "shipping_cost": shipping_total,
            "extras_total": extras_total,
            "grand_total": grand_total,
            "currency": obj.total_price_items.currency.code
            if obj.total_price_items
            else "EUR",
            "paid_amount": obj.paid_amount.amount if obj.paid_amount else 0,
            "remaining_amount": max(
                grand_total
                - (obj.paid_amount.amount if obj.paid_amount else 0),
                0,
            ),
        }

    @extend_schema_field(
        {
            "type": "object",
            "nullable": True,
            "properties": {
                "tracking_number": {
                    "type": "string",
                    "nullable": True,
                },
                "shipping_carrier": {
                    "type": "string",
                    "nullable": True,
                },
                "has_tracking": {"type": "boolean"},
                "estimated_delivery": {
                    "type": "string",
                    "nullable": True,
                },
                "tracking_url": {
                    "type": "string",
                    "nullable": True,
                },
            },
        }
    )
    def get_tracking_details(self, obj) -> dict | None:
        return {
            "tracking_number": obj.tracking_number,
            "shipping_carrier": obj.shipping_carrier,
            "has_tracking": bool(obj.tracking_number),
            "estimated_delivery": "",  # @TODO - Would be calculated based on shipping method
            "tracking_url": f"https://track.carrier.com/{obj.tracking_number}"
            if obj.tracking_number
            else None,
        }

    class Meta(OrderSerializer.Meta):
        fields = (
            *OrderSerializer.Meta.fields,
            "items",
            "country",
            "region",
            "pay_way",
            "order_timeline",
            "pricing_breakdown",
            "tracking_details",
            "phone",
            "mobile_phone",
            "document_type",
            "payment_id",
            "payment_status",
            "payment_method",
            "tracking_number",
            "shipping_carrier",
            "customer_full_name",
            "is_completed",
            "is_canceled",
            "full_address",
        )
        read_only_fields = (
            *OrderSerializer.Meta.read_only_fields,
            "order_timeline",
            "pricing_breakdown",
            "tracking_details",
        )


class OrderWriteSerializer(serializers.ModelSerializer[Order]):
    items = OrderItemCreateSerializer(many=True)
    paid_amount = MoneyField(max_digits=11, decimal_places=2, required=False)
    shipping_price = MoneyField(max_digits=11, decimal_places=2)
    total_price_items = MoneyField(
        max_digits=11, decimal_places=2, read_only=True
    )
    total_price_extra = MoneyField(
        max_digits=11, decimal_places=2, read_only=True
    )
    phone = PhoneNumberField()
    mobile_phone = PhoneNumberField(required=False, allow_blank=True)
    payment_id = serializers.CharField(required=False)
    payment_status = serializers.CharField(required=False)
    payment_method = serializers.CharField(required=False)

    def validate_items(self, value: list[dict]) -> list[dict]:
        if not value:
            raise serializers.ValidationError(
                _("At least one item is required.")
            )

        for item_data in value:
            if item_data.get("quantity", 0) <= 0:
                raise serializers.ValidationError(
                    _("Item quantity must be greater than zero.")
                )

        return value

    def validate_email(self, value: str) -> str:
        if not value:
            raise serializers.ValidationError(_("Email is required."))

        email_domain = value.split("@")[-1]
        if is_disposable_domain(email_domain):
            raise serializers.ValidationError(
                _("Try using a different email address.")
            )
        return value

    def validate(self, data):
        items_data = data.get("items", [])
        for item_data in items_data:
            product_id = item_data.get("product")
            quantity = item_data.get("quantity", 0)

            try:
                if hasattr(product_id, "id"):
                    product = product_id
                else:
                    product = Product.objects.get(id=product_id)

                if not product.active:
                    raise serializers.ValidationError(
                        _(
                            "Product with id '{product_name}' is not available."
                        ).format(
                            product_name=product.safe_translation_getter(
                                "name", any_language=True
                            )
                        )
                    )

                if product.stock < quantity:
                    raise serializers.ValidationError(
                        _(
                            "Not enough stock for '{product_name}'. Available: {product_stock}, Requested: {quantity}"
                        ).format(
                            product_name=product.safe_translation_getter(
                                "name", any_language=True
                            ),
                            product_stock=product.stock,
                            quantity=quantity,
                        )
                    )
            except Product.DoesNotExist:
                raise serializers.ValidationError(
                    _("Product with id '{product_id}' does not exist.").format(
                        product_id=product_id
                    )
                ) from None

        return data

    def create(self, validated_data):
        items_data = validated_data.pop("items")

        order = Order.objects.create(**validated_data)

        for item_data in items_data:
            product = item_data.get("product")
            item_to_create = item_data.copy()
            item_to_create["price"] = product.final_price
            OrderItem.objects.create(order=order, **item_to_create)

        order.paid_amount = order.calculate_order_total_amount()
        order.save(update_fields=["paid_amount"])

        order_created.send(sender=Order, order=order)

        return order

    def update(self, instance, validated_data):
        items_data = validated_data.pop("items", None)

        for attr, value in validated_data.items():
            setattr(instance, attr, value)
        instance.save()

        if items_data is not None:
            instance.items.all().delete()

            for item_data in items_data:
                product = item_data.get("product")
                item_to_create = item_data.copy()
                item_to_create["price"] = product.final_price
                OrderItem.objects.create(order=instance, **item_to_create)

        return instance

    class Meta:
        model = Order
        fields = (
            "user",
            "country",
            "region",
            "floor",
            "location_type",
            "street",
            "street_number",
            "pay_way",
            "status",
            "first_name",
            "last_name",
            "email",
            "zipcode",
            "place",
            "city",
            "phone",
            "mobile_phone",
            "paid_amount",
            "customer_notes",
            "items",
            "shipping_price",
            "total_price_items",
            "total_price_extra",
            "document_type",
            "payment_id",
            "payment_status",
            "payment_method",
            "tracking_number",
            "shipping_carrier",
        )


class AddTrackingSerializer(serializers.Serializer):
    tracking_number = serializers.CharField(max_length=100)
    shipping_carrier = serializers.CharField(max_length=50)


class UpdateStatusSerializer(serializers.Serializer):
    status = serializers.ChoiceField(choices=OrderStatus.choices)



# Generated by Django 5.0.3 on 2024-03-31 12:31
import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0007_alter_product_category_alter_product_vat_and_more"),
        ("vat", "0002_vat_vat_created_at_idx_vat_vat_updated_at_idx"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["created_at"], name="product_created_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["updated_at"], name="product_updated_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["price"], name="product_pro_price_9daef3_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["stock"], name="product_pro_stock_c26e64_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["discount_percent"],
                name="product_pro_discoun_63d416_btree",
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["hits"], name="product_pro_hits_19ae26_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["weight"], name="product_pro_weight_bf3d6d_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["final_price"], name="product_pro_final_p_0230ad_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["discount_value"],
                name="product_pro_discoun_c1c630_btree",
            ),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["price_save_percent"],
                name="product_pro_price_s_7a19fc_btree",
            ),
        ),
        migrations.AddIndex(
            model_name="productcategory",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["created_at"], name="productcategory_created_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productcategory",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["updated_at"], name="productcategory_updated_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productcategory",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["sort_order"], name="productcategory_sort_order_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productfavourite",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["created_at"], name="productfavourite_created_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productfavourite",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["updated_at"], name="productfavourite_updated_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productimage",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["created_at"], name="productimage_created_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productimage",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["updated_at"], name="productimage_updated_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productimage",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["sort_order"], name="productimage_sort_order_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productimage",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["product", "is_main"],
                name="product_pro_product_5068ba_btree",
            ),
        ),
        migrations.AddIndex(
            model_name="productreview",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["created_at"], name="productreview_created_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productreview",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["updated_at"], name="productreview_updated_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productreview",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["published_at"], name="productreview_published_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productreview",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["is_published"], name="productreview_is_published_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="productreview",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["status"], name="product_pro_status_8c2214_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="productreview",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["rate"], name="product_pro_rate_b78d32_btree"
            ),
        ),
    ]



import os

from django.conf import settings
from django.core.files.storage import default_storage
from django.test import TestCase

from country.factories import CountryFactory

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE


class CountryModelTestCase(TestCase):
    def setUp(self):
        self.country = CountryFactory(
            alpha_2="GR",
            alpha_3="GRC",
            iso_cc=300,
            phone_code=30,
            num_regions=0,
        )

    def test_fields(self):
        self.assertEqual(self.country.alpha_2, "GR")
        self.assertEqual(self.country.alpha_3, "GRC")
        self.assertEqual(self.country.iso_cc, 300)
        self.assertEqual(self.country.phone_code, 30)
        self.assertTrue(default_storage.exists(self.country.image_flag.path))

    def test_str_representation(self):
        self.assertEqual(
            str(self.country),
            self.country.safe_translation_getter("name") or "",
        )

    def test_get_ordering_queryset(self):
        queryset = self.country.get_ordering_queryset()
        self.assertTrue(queryset.exists())
        self.assertTrue(self.country in queryset)

    def test_main_image_path(self):
        expected_filename = f"media/uploads/country/{os.path.basename(self.country.image_flag.name)}"
        self.assertEqual(self.country.main_image_path, expected_filename)



# Generated by Django 5.2.4 on 2025-07-31 08:19

import django.contrib.postgres.indexes
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0024_remove_productcategory_menu_image_one_and_more'),
        ('vat', '0006_rename_vat_created_at_idx_vat_created_at_ix_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='product',
            name='product_product_code_ix',
        ),
        migrations.RemoveField(
            model_name='historicalproduct',
            name='product_code',
        ),
        migrations.RemoveField(
            model_name='product',
            name='product_code',
        ),
        migrations.AddField(
            model_name='historicalproduct',
            name='sku',
            field=models.CharField(db_index=True, default=uuid.uuid4, max_length=100, verbose_name='SKU'),
        ),
        migrations.AddField(
            model_name='product',
            name='sku',
            field=models.CharField(default=uuid.uuid4, max_length=100, unique=True, verbose_name='SKU'),
        ),
        migrations.AddIndex(
            model_name='product',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['sku'], name='product_sku_ix'),
        ),
    ]



from unittest.mock import Mock, patch

import factory
from django.db import models
from django.db.models.fields.related import ForeignKey, OneToOneField
from django.test import TestCase

from core.utils.dependencies import (
    DependencyNode,
    CircularDependencyError,
    DependencyAnalyzer,
    FactoryOrchestrator,
    get_factory_execution_order,
    analyze_factory_dependencies,
)


class MockUser(models.Model):
    class Meta:
        app_label = "test"


class MockProfile(models.Model):
    user = models.OneToOneField(MockUser, on_delete=models.CASCADE)

    class Meta:
        app_label = "test"


class MockPost(models.Model):
    author = models.ForeignKey(MockUser, on_delete=models.CASCADE)

    class Meta:
        app_label = "test"


class MockComment(models.Model):
    post = models.ForeignKey(MockPost, on_delete=models.CASCADE)
    author = models.ForeignKey(MockUser, on_delete=models.CASCADE)

    class Meta:
        app_label = "test"


class MockUserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MockUser

    execution_priority = 0


class MockProfileFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MockProfile

    user = factory.SubFactory(MockUserFactory)
    execution_priority = 1


class MockPostFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MockPost

    author = factory.SubFactory(MockUserFactory)
    execution_priority = 2


class MockCommentFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MockComment

    post = factory.SubFactory(MockPostFactory)
    author = factory.SubFactory(MockUserFactory)
    execution_priority = 3


class MockConditionalFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MockUser

    conditional_depends_on = {
        "admin_required": ["MockUserFactory"],
        "has_inventory": ["MockProfileFactory"],
    }
    execution_priority = 5


class MockExplicitDepsFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MockUser

    depends_on = ["MockUserFactory", "MockProfileFactory"]
    business_logic_deps = ["MockPostFactory"]
    execution_priority = 4
    locale_aware = True
    business_rules_enabled = True


class TestDependencyNode(TestCase):
    def setUp(self):
        self.node = DependencyNode(MockUserFactory)

    def test_node_initialization(self):
        self.assertEqual(self.node.factory_class, MockUserFactory)
        self.assertEqual(self.node.model_class, MockUser)
        self.assertEqual(self.node.dependencies, set())
        self.assertEqual(self.node.dependents, set())
        self.assertEqual(self.node.level, 0)
        self.assertEqual(self.node.execution_priority, 0)

    def test_node_properties(self):
        self.assertEqual(self.node.name, "MockUserFactory")
        self.assertEqual(self.node.model_name, "MockUser")

    def test_add_dependency(self):
        profile_node = DependencyNode(MockProfileFactory)
        self.node.add_dependency(profile_node, "FK")

        self.assertIn(profile_node, self.node.dependencies)
        self.assertIn(self.node, profile_node.dependents)
        self.assertEqual(self.node.dependency_types["MockProfileFactory"], "FK")

    def test_conditional_dependencies_parsing(self):
        conditional_node = DependencyNode(MockConditionalFactory)

        self.assertEqual(
            conditional_node.conditional_dependencies,
            {
                "admin_required": ["MockUserFactory"],
                "has_inventory": ["MockProfileFactory"],
            },
        )

    def test_explicit_dependencies_parsing(self):
        explicit_node = DependencyNode(MockExplicitDepsFactory)

        self.assertEqual(
            explicit_node.explicit_dependencies,
            ["MockUserFactory", "MockProfileFactory"],
        )
        self.assertEqual(explicit_node.business_logic_deps, ["MockPostFactory"])
        self.assertEqual(explicit_node.execution_priority, 4)
        self.assertTrue(explicit_node.locale_aware)
        self.assertTrue(explicit_node.business_rules_enabled)

    def test_evaluate_condition(self):
        conditional_node = DependencyNode(MockConditionalFactory)

        context_admin = {"create_admin_users": True}
        self.assertTrue(
            conditional_node._evaluate_condition(
                "admin_required", context_admin
            )
        )

        context_no_admin = {"create_admin_users": False}
        self.assertFalse(
            conditional_node._evaluate_condition(
                "admin_required", context_no_admin
            )
        )

        context_inventory = {"create_inventory": True}
        self.assertTrue(
            conditional_node._evaluate_condition(
                "has_inventory", context_inventory
            )
        )

        context_no_inventory = {"create_inventory": False}
        self.assertFalse(
            conditional_node._evaluate_condition(
                "has_inventory", context_no_inventory
            )
        )

    def test_get_conditional_deps_for_context(self):
        conditional_node = DependencyNode(MockConditionalFactory)

        context = {"create_admin_users": True, "create_inventory": False}
        deps = conditional_node.get_conditional_deps_for_context(context)
        self.assertEqual(deps, ["MockUserFactory"])

        context = {"create_admin_users": False, "create_inventory": True}
        deps = conditional_node.get_conditional_deps_for_context(context)
        self.assertEqual(deps, ["MockProfileFactory"])

    def test_string_representations(self):
        self.assertEqual(
            str(self.node), "MockUserFactory (level 0, priority 0)"
        )
        self.assertEqual(
            repr(self.node),
            "DependencyNode(MockUserFactory, level=0, priority=0)",
        )


class TestCircularDependencyError(TestCase):
    def test_circular_dependency_error(self):
        node1 = DependencyNode(MockUserFactory)
        node2 = DependencyNode(MockProfileFactory)
        cycle = [node1, node2, node1]

        error = CircularDependencyError(cycle)
        self.assertEqual(error.cycle, cycle)
        expected_message = "Circular dependency detected: MockUserFactory -> MockProfileFactory -> MockUserFactory"
        self.assertEqual(str(error), expected_message)


class TestDependencyAnalyzer(TestCase):
    def setUp(self):
        self.analyzer = DependencyAnalyzer()

    def test_analyzer_initialization(self):
        self.assertEqual(self.analyzer.nodes, {})
        self.assertEqual(self.analyzer.model_to_factory, {})

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_create_nodes(self, mock_is_translation):
        mock_is_translation.return_value = False

        factories = [MockUserFactory, MockProfileFactory]
        self.analyzer._create_nodes(factories)

        self.assertIn("MockUserFactory", self.analyzer.nodes)
        self.assertIn("MockProfileFactory", self.analyzer.nodes)
        self.assertEqual(len(self.analyzer.nodes), 2)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_create_nodes_skips_translation_factories(
        self, mock_is_translation
    ):
        def side_effect(factory_class):
            return factory_class.__name__ == "TranslationFactory"

        mock_is_translation.side_effect = side_effect

        class TranslationFactory(factory.django.DjangoModelFactory):
            class Meta:
                model = MockUser

        factories = [MockUserFactory, TranslationFactory]
        self.analyzer._create_nodes(factories)

        self.assertIn("MockUserFactory", self.analyzer.nodes)
        self.assertNotIn("TranslationFactory", self.analyzer.nodes)

    def test_should_skip_dependency(self):
        mock_field = Mock(spec=ForeignKey)
        mock_field.null = True
        mock_field.default = "some_default"

        self.assertTrue(
            self.analyzer._should_skip_dependency(mock_field, MockUser)
        )

        system_model = Mock()
        system_model.__name__ = "User"
        self.assertTrue(
            self.analyzer._should_skip_dependency(mock_field, system_model)
        )

        skip_model = Mock()
        skip_model.__name__ = "TestModel"
        skip_model._meta.app_label = "auth"
        self.assertTrue(
            self.analyzer._should_skip_dependency(mock_field, skip_model)
        )

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_analyze_model_dependencies(self, mock_is_translation):
        mock_is_translation.return_value = False

        self.analyzer._create_nodes([MockUserFactory, MockProfileFactory])

        profile_node = self.analyzer.nodes["MockProfileFactory"]

        user_field = Mock(spec=OneToOneField)
        user_field.related_model = MockUser
        user_field.name = "user"
        user_field.null = False
        user_field.default = models.NOT_PROVIDED

        with patch.object(
            MockProfile._meta, "get_fields", return_value=[user_field]
        ):
            self.analyzer._analyze_model_dependencies(profile_node)

        user_node = self.analyzer.nodes["MockUserFactory"]
        self.assertIn(user_node, profile_node.dependencies)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_analyze_explicit_dependencies(self, mock_is_translation):
        mock_is_translation.return_value = False

        factories = [
            MockUserFactory,
            MockProfileFactory,
            MockExplicitDepsFactory,
        ]
        self.analyzer._create_nodes(factories)

        explicit_node = self.analyzer.nodes["MockExplicitDepsFactory"]
        context = {}

        self.analyzer._analyze_explicit_dependencies(explicit_node, context)

        user_node = self.analyzer.nodes["MockUserFactory"]
        profile_node = self.analyzer.nodes["MockProfileFactory"]

        self.assertIn(user_node, explicit_node.dependencies)
        self.assertIn(profile_node, explicit_node.dependencies)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_detect_circular_dependencies(self, mock_is_translation):
        mock_is_translation.return_value = False

        self.analyzer._create_nodes([MockUserFactory, MockProfileFactory])

        user_node = self.analyzer.nodes["MockUserFactory"]
        profile_node = self.analyzer.nodes["MockProfileFactory"]

        user_node.add_dependency(profile_node)
        profile_node.add_dependency(user_node)

        with self.assertRaises(CircularDependencyError):
            self.analyzer._detect_circular_dependencies()

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_topological_sort(self, mock_is_translation):
        mock_is_translation.return_value = False

        self.analyzer._create_nodes([MockUserFactory, MockProfileFactory])

        user_node = self.analyzer.nodes["MockUserFactory"]
        profile_node = self.analyzer.nodes["MockProfileFactory"]
        profile_node.add_dependency(user_node)

        sorted_nodes = self.analyzer._topological_sort()

        user_index = next(
            i
            for i, node in enumerate(sorted_nodes)
            if node.name == "MockUserFactory"
        )
        profile_index = next(
            i
            for i, node in enumerate(sorted_nodes)
            if node.name == "MockProfileFactory"
        )

        self.assertLess(user_index, profile_index)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_get_dependency_report(self, mock_is_translation):
        mock_is_translation.return_value = False

        self.analyzer._create_nodes([MockUserFactory, MockProfileFactory])

        report = self.analyzer.get_dependency_report()

        self.assertIn("Dependency Analysis Report", report)
        self.assertIn("MockUserFactory", report)
        self.assertIn("MockProfileFactory", report)
        self.assertIn("Statistics:", report)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    @patch("core.utils.dependencies.logger")
    def test_analyze_factory_dependencies_full_flow(
        self, mock_logger, mock_is_translation
    ):
        mock_is_translation.return_value = False

        factories = [MockUserFactory, MockProfileFactory, MockPostFactory]

        with (
            patch.object(
                MockProfile._meta, "get_fields"
            ) as mock_profile_fields,
            patch.object(MockPost._meta, "get_fields") as mock_post_fields,
            patch.object(MockUser._meta, "get_fields", return_value=[]),
        ):
            user_field = Mock(spec=OneToOneField)
            user_field.related_model = MockUser
            user_field.name = "user"
            user_field.null = False
            user_field.default = models.NOT_PROVIDED
            mock_profile_fields.return_value = [user_field]

            author_field = Mock(spec=ForeignKey)
            author_field.related_model = MockUser
            author_field.name = "author"
            author_field.null = False
            author_field.default = models.NOT_PROVIDED
            mock_post_fields.return_value = [author_field]

            result = self.analyzer.analyze_factory_dependencies(factories)

            self.assertIsInstance(result, list)
            self.assertTrue(
                all(isinstance(node, DependencyNode) for node in result)
            )

            self.assertEqual(result[0].name, "MockUserFactory")


class TestFactoryOrchestrator(TestCase):
    def setUp(self):
        self.orchestrator = FactoryOrchestrator()

    def test_orchestrator_initialization(self):
        self.assertIsInstance(self.orchestrator.analyzer, DependencyAnalyzer)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    @patch("core.utils.dependencies.logger")
    def test_get_execution_order(self, mock_logger, mock_is_translation):
        mock_is_translation.return_value = False

        factories = [
            MockProfileFactory,
            MockUserFactory,
        ]

        with (
            patch.object(MockProfile._meta, "get_fields") as mock_fields,
            patch.object(MockUser._meta, "get_fields", return_value=[]),
        ):
            user_field = Mock(spec=OneToOneField)
            user_field.related_model = MockUser
            user_field.name = "user"
            user_field.null = False
            user_field.default = models.NOT_PROVIDED
            mock_fields.return_value = [user_field]

            result = self.orchestrator.get_execution_order(factories)

            factory_names = [f.__name__ for f in result]
            user_index = factory_names.index("MockUserFactory")
            profile_index = factory_names.index("MockProfileFactory")
            self.assertLess(user_index, profile_index)

    @patch(
        "core.utils.dependencies.TranslationUtilities.is_translation_factory"
    )
    def test_get_dependency_report(self, mock_is_translation):
        mock_is_translation.return_value = False

        factories = [MockUserFactory, MockProfileFactory]
        self.orchestrator.get_execution_order(factories)

        report = self.orchestrator.get_dependency_report()
        self.assertIsInstance(report, str)
        self.assertIn("Dependency Analysis Report", report)


class TestModuleFunctions(TestCase):
    @patch("core.utils.dependencies.FactoryOrchestrator.get_execution_order")
    def test_get_factory_execution_order(self, mock_get_order):
        factory_classes = [MockUserFactory, MockProfileFactory]
        expected_order = [MockUserFactory, MockProfileFactory]
        mock_get_order.return_value = expected_order

        result = get_factory_execution_order(factory_classes)

        self.assertEqual(result, expected_order)
        mock_get_order.assert_called_once_with(factory_classes)

    @patch("core.utils.dependencies.FactoryOrchestrator.get_dependency_report")
    @patch("core.utils.dependencies.FactoryOrchestrator.get_execution_order")
    def test_analyze_factory_dependencies_function(
        self, mock_get_order, mock_get_report
    ):
        factory_classes = [MockUserFactory, MockProfileFactory]
        expected_report = "Test report"
        mock_get_report.return_value = expected_report

        result = analyze_factory_dependencies(factory_classes)

        self.assertEqual(result, expected_report)
        mock_get_order.assert_called_once_with(factory_classes)
        mock_get_report.assert_called_once()



from unittest.mock import Mock

from django.test import TestCase

from product.filters.product import ProductFilter
from product.serializers.image import ProductImageSerializer
from product.serializers.product import (
    ProductDetailSerializer,
    ProductSerializer,
    ProductWriteSerializer,
)
from product.serializers.review import ProductReviewSerializer
from product.views.product import ProductViewSet
from tag.serializers import TagSerializer


class ProductViewSetTestCase(TestCase):
    def setUp(self):
        self.viewset = ProductViewSet()

        self.mock_request = Mock()
        self.mock_request.query_params.get.return_value = None
        self.mock_request.user.is_authenticated = False
        self.viewset.request = self.mock_request

        mock_queryset = Mock()
        mock_queryset.with_all_annotations.return_value = mock_queryset
        self.viewset.get_queryset = Mock(return_value=mock_queryset)

    def test_list_action_serializer(self):
        self.viewset.action = "list"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductSerializer)

    def test_retrieve_action_serializer(self):
        self.viewset.action = "retrieve"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductDetailSerializer)

    def test_update_action_serializer(self):
        self.viewset.action = "update"
        serializer_class = self.viewset.get_serializer_class()
        self.assertEqual(serializer_class, ProductDetailSerializer)

    def test_request_serializer_class(self):
        self.viewset.action = "create"
        request_serializer = self.viewset.get_request_serializer()
        self.assertEqual(request_serializer, ProductWriteSerializer)

    def test_response_serializer_class(self):
        self.viewset.action = "create"
        response_serializer = self.viewset.get_response_serializer()
        self.assertEqual(response_serializer, ProductDetailSerializer)

        self.viewset.action = "update"
        response_serializer = self.viewset.get_response_serializer()
        self.assertEqual(response_serializer, ProductDetailSerializer)

    def test_custom_action_serializers(self):
        self.viewset.action = "reviews"
        serializer_class = self.viewset.get_serializer_class()
        self.assertIsNotNone(serializer_class)

        self.viewset.action = "images"
        serializer_class = self.viewset.get_serializer_class()
        self.assertIsNotNone(serializer_class)

        self.viewset.action = "tags"
        serializer_class = self.viewset.get_serializer_class()
        self.assertIsNotNone(serializer_class)

    def test_get_serializer_for_schema(self):
        schema_info = self.viewset.get_serializer_for_schema("create")
        self.assertEqual(schema_info["request"], ProductWriteSerializer)
        self.assertEqual(schema_info["response"], ProductDetailSerializer)

        schema_info = self.viewset.get_serializer_for_schema("list")
        self.assertEqual(schema_info["request"], ProductSerializer)
        self.assertEqual(schema_info["response"], ProductSerializer)

        original_action = self.viewset.action
        self.viewset.get_serializer_for_schema("create")
        self.assertEqual(self.viewset.action, original_action)

    def test_ordering_fields_configuration(self):
        expected_fields = [
            "price",
            "created_at",
            "active",
            "availability_priority",
            "view_count",
            "stock",
        ]
        self.assertEqual(
            set(self.viewset.ordering_fields), set(expected_fields)
        )

    def test_search_fields_configuration(self):
        expected_fields = [
            "translations__name",
            "translations__description",
            "slug",
        ]
        self.assertEqual(set(self.viewset.search_fields), set(expected_fields))

    def test_default_ordering(self):
        self.assertEqual(
            self.viewset.ordering, ["-availability_priority", "id"]
        )

    def test_filterset_class_configuration(self):
        self.assertEqual(self.viewset.filterset_class, ProductFilter)

    def test_response_serializers_configuration(self):
        expected_response_serializers = {
            "create": ProductDetailSerializer,
            "list": ProductSerializer,
            "retrieve": ProductDetailSerializer,
            "update": ProductDetailSerializer,
            "partial_update": ProductDetailSerializer,
            "update_view_count": ProductDetailSerializer,
            "reviews": ProductReviewSerializer,
            "images": ProductImageSerializer,
            "tags": TagSerializer,
        }
        print("")
        self.assertEqual(
            self.viewset.response_serializers, expected_response_serializers
        )

    def test_queryset_optimization(self):
        queryset = self.viewset.get_queryset()
        self.assertIsNotNone(queryset)



import unittest

from core.utils.files import sanitize_filename


class TestSanitizeFilename(unittest.TestCase):
    def test_basic_filename(self):
        result = sanitize_filename("document.txt")
        self.assertEqual(result, "document.txt")

    def test_filename_with_spaces(self):
        result = sanitize_filename("my document.txt")
        self.assertEqual(result, "mydocument.txt")

    def test_filename_with_special_characters(self):
        result = sanitize_filename("file@#$%^&*()+={}[]|\\:;\"'<>?,/~`")
        self.assertEqual(result, "file")

    def test_filename_with_allowed_characters(self):
        result = sanitize_filename("My-File_Name.123.txt")
        self.assertEqual(result, "My-File_Name.123.txt")

    def test_filename_with_numbers(self):
        result = sanitize_filename("file123.txt")
        self.assertEqual(result, "file123.txt")

    def test_filename_with_uppercase_and_lowercase(self):
        result = sanitize_filename("MyFileName.TXT")
        self.assertEqual(result, "MyFileName.TXT")

    def test_filename_with_multiple_dots(self):
        result = sanitize_filename("file...name....txt")
        self.assertEqual(result, "file.name.txt")

    def test_filename_starting_with_dots(self):
        result = sanitize_filename("...filename.txt")
        self.assertEqual(result, "filename.txt")

    def test_filename_ending_with_dots(self):
        result = sanitize_filename("filename.txt...")
        self.assertEqual(result, "filename.txt")

    def test_filename_only_dots(self):
        result = sanitize_filename(".....")
        self.assertEqual(result, "")

    def test_empty_string(self):
        result = sanitize_filename("")
        self.assertEqual(result, "")

    def test_whitespace_only_string(self):
        result = sanitize_filename("   \t\n\r  ")
        self.assertEqual(result, "")

    def test_filename_length_under_limit(self):
        filename = "a" * 40 + ".txt"
        result = sanitize_filename(filename)
        self.assertEqual(result, filename)
        self.assertEqual(len(result), 44)

    def test_filename_length_exactly_at_limit(self):
        filename = "a" * 46 + ".txt"
        result = sanitize_filename(filename)
        self.assertEqual(result, filename)
        self.assertEqual(len(result), 50)

    def test_filename_length_over_limit(self):
        filename = "a" * 60 + ".txt"
        result = sanitize_filename(filename)
        self.assertEqual(len(result), 50)
        self.assertEqual(result, "a" * 50)

    def test_long_filename_with_extension(self):
        filename = (
            "very_long_filename_that_exceeds_the_fifty_character_limit.txt"
        )
        result = sanitize_filename(filename)
        self.assertEqual(len(result), 50)
        self.assertEqual(result, filename[:50])

    def test_unicode_characters(self):
        result = sanitize_filename("файл.txt")
        self.assertEqual(result, "txt")

    def test_unicode_emoji(self):
        result = sanitize_filename("file😀📁.txt")
        self.assertEqual(result, "file.txt")

    def test_chinese_characters(self):
        result = sanitize_filename("文件.txt")
        self.assertEqual(result, "txt")

    def test_accented_characters(self):
        result = sanitize_filename("résumé.pdf")
        self.assertEqual(result, "rsum.pdf")

    def test_filename_with_underscores(self):
        result = sanitize_filename("my_file_name.txt")
        self.assertEqual(result, "my_file_name.txt")

    def test_filename_with_hyphens(self):
        result = sanitize_filename("my-file-name.txt")
        self.assertEqual(result, "my-file-name.txt")

    def test_filename_mixed_separators(self):
        result = sanitize_filename("my-file_name.with-dots.txt")
        self.assertEqual(result, "my-file_name.with-dots.txt")

    def test_filename_only_extension(self):
        result = sanitize_filename(".txt")
        self.assertEqual(result, "txt")

    def test_filename_no_extension(self):
        result = sanitize_filename("filename")
        self.assertEqual(result, "filename")

    def test_filename_multiple_extensions(self):
        result = sanitize_filename("archive.tar.gz")
        self.assertEqual(result, "archive.tar.gz")

    def test_filename_with_path_separators(self):
        result = sanitize_filename("path/to/file.txt")
        self.assertEqual(result, "pathtofile.txt")

        result = sanitize_filename("path\\to\\file.txt")
        self.assertEqual(result, "pathtofile.txt")

    def test_filename_with_tabs_and_newlines(self):
        result = sanitize_filename("file\tname\n.txt")
        self.assertEqual(result, "filename.txt")

    def test_filename_with_null_bytes(self):
        result = sanitize_filename("file\x00name.txt")
        self.assertEqual(result, "filename.txt")

    def test_filename_with_control_characters(self):
        result = sanitize_filename("file\x01\x02\x03name.txt")
        self.assertEqual(result, "filename.txt")

    def test_complex_sanitization(self):
        result = sanitize_filename(
            "...My File@#$%^&*()Name...with spaces...and...dots....txt..."
        )
        self.assertEqual(result, "MyFileName.withspaces.and.dots.txt")

    def test_edge_case_only_special_characters(self):
        result = sanitize_filename("@#$%^&*()")
        self.assertEqual(result, "")

    def test_edge_case_only_dots_and_spaces(self):
        result = sanitize_filename("... ... ...")
        self.assertEqual(result, "")

    def test_preservation_of_case(self):
        result = sanitize_filename("MyFile.TXT")
        self.assertEqual(result, "MyFile.TXT")

    def test_numeric_filename(self):
        result = sanitize_filename("12345.678")
        self.assertEqual(result, "12345.678")

    def test_single_character_filename(self):
        result = sanitize_filename("a")
        self.assertEqual(result, "a")

    def test_single_dot_filename(self):
        result = sanitize_filename(".")
        self.assertEqual(result, "")

    def test_filename_with_repeated_special_chars(self):
        result = sanitize_filename("file!!!???###.txt")
        self.assertEqual(result, "file.txt")

    def test_filename_boundary_at_50_chars_with_dots(self):
        filename = "a" * 45 + "....." + ".txt"
        result = sanitize_filename(filename)
        expected = "a" * 45 + ".txt"
        self.assertEqual(result, expected)
        self.assertEqual(len(result), 49)

    def test_filename_with_dots_at_truncation_point(self):
        filename = "a" * 48 + "....txt"
        result = sanitize_filename(filename)
        self.assertEqual(len(result), 50)
        self.assertEqual(result, "a" * 48 + ".t")

    def test_real_world_filenames(self):
        test_cases = [
            ("My Document (1).pdf", "MyDocument1.pdf"),
            ("IMG_20231225_143052.jpg", "IMG_20231225_143052.jpg"),
            ("Report - Final Version.docx", "Report-FinalVersion.docx"),
            ("data_export_2023-12-25.csv", "data_export_2023-12-25.csv"),
            ("config.backup.json", "config.backup.json"),
            ("script.sh.bak", "script.sh.bak"),
        ]

        for input_filename, expected in test_cases:
            with self.subTest(filename=input_filename):
                result = sanitize_filename(input_filename)
                self.assertEqual(result, expected)

    def test_security_related_filenames(self):
        test_cases = [
            ("../../../etc/passwd", "etcpasswd"),
            ("..\\..\\windows\\system32", "windowssystem32"),
            (
                "file<script>alert('xss')</script>.txt",
                "filescriptalertxssscript.txt",
            ),
            ("file\x00.txt", "file.txt"),
            (
                "CON.txt",
                "CON.txt",
            ),
            ("file|pipe.txt", "filepipe.txt"),
        ]

        for input_filename, expected in test_cases:
            with self.subTest(filename=input_filename):
                result = sanitize_filename(input_filename)
                self.assertEqual(result, expected)

    def test_function_return_type(self):
        result = sanitize_filename("test.txt")
        self.assertIsInstance(result, str)

        result = sanitize_filename("")
        self.assertIsInstance(result, str)

        result = sanitize_filename("@#$%")
        self.assertIsInstance(result, str)

    def test_function_idempotency(self):
        filename = "My File@#$%^&*()Name.txt"
        result1 = sanitize_filename(filename)
        result2 = sanitize_filename(result1)
        result3 = sanitize_filename(result2)

        self.assertEqual(result1, result2)
        self.assertEqual(result2, result3)
        self.assertEqual(result1, "MyFileName.txt")

    def test_function_with_none_input(self):
        with self.assertRaises(TypeError):
            sanitize_filename(None)

    def test_function_with_integer_input(self):
        with self.assertRaises(TypeError):
            sanitize_filename(123)

    def test_function_with_list_input(self):
        with self.assertRaises(TypeError):
            sanitize_filename(["file.txt"])



# Generated by Django 5.1.2 on 2024-10-26 08:10
import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("blog", "0022_alter_blogposttranslation_master"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddIndex(
            model_name="blogpost",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["category"], name="blog_blogpo_categor_4d71d3_btree"
            ),
        ),
        migrations.AddIndex(
            model_name="blogpost",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["author"], name="blog_blogpo_author__753b9d_btree"
            ),
        ),
    ]
